.packageName <- "RcmdrPlugin.HH"
"BoxCox" <-
function(){
    require("car")
    initializeDialog(title="Box-Cox Transformations")
    variablesBox <- variableListBox(top, Numeric(), selectmode="multiple",
        title="Select variables (one or more)")
    onOK <- function(){
        variables <- getSelection(variablesBox)
        if (length(variables) < 1) {
            errorCondition(recall=BoxCox,
              message="You must select one or more variables.")
            return()
            }
        closeDialog()
        command <- paste("box.cox.powers(na.omit(cbind(",
            paste(paste(variables, "=", ActiveDataSet(), "$", variables, sep=""),
                collapse=", "), ")))", sep="")
        doItAndPrint(command)
        tkfocus(CommanderWindow())
        }
    OKCancelHelp(helpSubject="box.cox.powers")
    tkgrid(getFrame(variablesBox), sticky="nw")
    tkgrid(buttonsFrame, sticky="w")
    dialogSuffix(rows=2, columns=1)
    }

ChisqHypothesesPlot <- function(){
    initializeDialog(title=gettextRcmdr("Chisq Distribution"))
    degfreeVar <- tclVar("1")
    degfreeEntry <- tkentry(top, width="6", textvariable=degfreeVar)
    alphaLeftVar <- tclVar("")
    alphaLeftEntry <- tkentry(top, width="6", textvariable=alphaLeftVar)
    alphaRightVar <- tclVar(".05")
    alphaRightEntry <- tkentry(top, width="6", textvariable=alphaRightVar)
    obsValueVar <- tclVar("")
    obsValueEntry <- tkentry(top, width="6", textvariable=obsValueVar)
    xmaxVar <- tclVar("")
    xmaxEntry <- tkentry(top, width="6", textvariable=xmaxVar)
    ymaxVar <- tclVar("")
    ymaxEntry <- tkentry(top, width="6", textvariable=ymaxVar)
    onOK <- function(){
      closeDialog()
      degfree <- as.numeric(tclvalue(degfreeVar))
      alphaLeft <- as.numeric(tclvalue(alphaLeftVar))
      alphaRight <- as.numeric(tclvalue(alphaRightVar))
      obsValue <- as.numeric(tclvalue(obsValueVar))
      xmax <- as.numeric(tclvalue(xmaxVar))
      ymax <- as.numeric(tclvalue(ymaxVar))
      
      command <- "old.par <- par(oma=c(1,1,0,5))"
      justDoIt(command)
      logger(command)
      
      command.xlim <- if (!is.na(xmax)) 
        paste(", xlim=c(0, ", xmax, ")", sep="")
      else ""
      
      command.ylim <- if (!is.na(ymax)) 
        paste(", ylim=c(0, ", ymax, ")", sep="")
      else ""
      
      command <- "chisq.setup("
      if (!is.na(degfree))
        command <- paste(command, "df=", degfree, sep="")
      else
        degfree <-1

      command <- paste(command,
                       command.xlim,
                       command.ylim,
                       sep="")

      command <- paste(command, ")", sep="")
      justDoIt(command)
      logger(command)

      command.alpha <- ", alpha=c("
      if (!is.na(alphaLeft)) command.alpha <-
        paste(command.alpha, alphaLeft, ", ", sep="")
      command.alpha <-  paste(command.alpha, alphaRight, ")", sep="")

      shade <- if (!is.na(alphaLeft)) "outside" else "right"

      
      command <- paste("chisq.curve(",
                       "df=", degfree,
                       command.alpha, 
                       ", shade='", shade,
                       "', col='blue')", sep="")
      doItAndPrint(command)

      ## Observed Value
      if (!is.na(obsValue)) {
        command <- paste("chisq.observed(", obsValue, ", col='green'",
                       ", df=", degfree,
                       ")",  sep="")
        doItAndPrint(command)
      }

      command <- "par(old.par)"
      justDoIt(command)
      logger(command)
      
      tkfocus(CommanderWindow())
    }
    OKCancelHelp(helpSubject="dnorm")
    tkgrid(tklabel(top, text=gettextRcmdr("df (degrees of freedom)")), degfreeEntry, sticky="e")
    tkgrid(tklabel(top, text=gettextRcmdr("left alpha")), alphaLeftEntry, sticky="e")
    tkgrid(tklabel(top, text=gettextRcmdr("right alpha")), alphaRightEntry, sticky="e")
    tkgrid(tklabel(top, text=gettextRcmdr("Observed Value")), obsValueEntry, sticky="e")
    tkgrid(tklabel(top, text=gettextRcmdr("Chisq max (right-hand side)")), xmaxEntry, sticky="e")
    tkgrid(tklabel(top, text=gettextRcmdr("f(Chisq) max (top side)")), ymaxEntry, sticky="e")
    tkgrid(buttonsFrame, columnspan=1, sticky="w")
    tkgrid.configure(degfreeEntry, sticky="w")
    tkgrid.configure(alphaLeftEntry, sticky="w")
    tkgrid.configure(alphaRightEntry, sticky="w")
    tkgrid.configure(obsValueEntry, sticky="w")
    tkgrid.configure(xmaxEntry, sticky="w")
    tkgrid.configure(ymaxEntry, sticky="w")
    dialogSuffix(rows=8, columns=1, focus=degfreeEntry)
    }

## source("~/HH-R.package/RcmdrPlugin.HH/R/ChisqHypothesesPlot.R")
## source("~/HH-R.package/HH/R/chisq.curve.R")
"DotplottbRcmdr" <-
function() {
    initializeDialog(title=gettextRcmdr("dotplot with tiebreakers"))
    factorBox <- variableListBox(top, Factors(), title=gettextRcmdr("Factors (pick zero or one)"), selectmode="single", initialSelection=-1)
    responseBox <- variableListBox(top, Numeric(), title=gettextRcmdr("~ Response Variable (pick one)"))
    groupBox <- variableListBox(top, Factors(), title=gettextRcmdr(" | Groups (pick zero or more)"), selectmode="multiple", initialSelection=-1)

    scalarsFrame <- tkframe(top)

    factor.jitterVar <- tclVar(".5")
    factor.jitterEntry <- tkentry(scalarsFrame, width="6", textvariable=factor.jitterVar)

    cexVar <- tclVar("1")
    cexEntry <- tkentry(scalarsFrame, width="6", textvariable=cexVar)

    pchVar <- tclVar("16")
    pchEntry <- tkentry(scalarsFrame, width="6", textvariable=pchVar)
    
    layxVar <- tclVar("")
    layxEntry <- tkentry(scalarsFrame, width="6", textvariable=layxVar)
    
    layyVar <- tclVar("")
    layyEntry <- tkentry(scalarsFrame, width="6", textvariable=layyVar)
    
    betxVar <- tclVar("0")
    betxEntry <- tkentry(scalarsFrame, width="6", textvariable=betxVar)
    
    betyVar <- tclVar("1")
    betyEntry <- tkentry(scalarsFrame, width="6", textvariable=betyVar)

    altxVar <- tclVar("1")
    altxEntry <- tkentry(scalarsFrame, width="6", textvariable=altxVar)
    
    altyVar <- tclVar("1")
    altyEntry <- tkentry(scalarsFrame, width="6", textvariable=altyVar)

    onOK <- function() {
        factor <- getSelection(factorBox)
        groups <- getSelection(groupBox)
        response <- getSelection(responseBox)

        factor.jitter  <- as.numeric(tclvalue(factor.jitterVar))
        cex            <- as.numeric(tclvalue(cexVar))
        old.warn <- options(warn=-1)
        pch            <- as.numeric(tclvalue(pchVar))
        if (is.na(pch)) pch <- paste("\'", tclvalue(pchVar), "\'", sep="")
        options(old.warn)
        layx           <- as.numeric(tclvalue(layxVar))
        layy           <- as.numeric(tclvalue(layyVar))
        betx           <- as.numeric(tclvalue(betxVar))
        bety           <- as.numeric(tclvalue(betyVar))
        altx           <- as.numeric(tclvalue(altxVar))
        alty           <- as.numeric(tclvalue(altyVar))
        
        closeDialog()
        if (0 == length(response)) {
            errorCondition(recall=DotplottbRcmdr,
                           message=gettextRcmdr("No response variable selected."))
            return()
            }
        .activeDataSet <- ActiveDataSet()
        groups.formula <- paste(groups, collapse=' + ')
        if (nchar(groups.formula) > 0) {
          groups.formula <- paste(" | ", groups.formula, sep="")
          freq.formula <- paste("with(get(.activeDataSet),",
                                "tapply(",
                                "get(response)",
                                ", data.frame(factor(get(response))",
                                if (length(factor)==1) ", get(factor)",
                                if (length(groups)>0) ", get(paste(groups, collapse=','))",
                                "), length))")
          freq <- eval(parse(text=freq.formula))
          max.freq <- max(freq, na.rm=TRUE)
          groups.formula <- paste(groups.formula,
                                  ", max.freq=", max.freq, sep="")
        }
        if (length(factor)==0) {
          n.response <- eval(parse(text='length(get(.activeDataSet)[[response]])'))
          factor <- paste("rep('', nrow(", .activeDataSet, "))", sep="")
        }
        dptb.command <- paste("dotplot(", factor, " ~ ",  response,
                              groups.formula,
                              ", data=", .activeDataSet, ",", sep="")

        if (is.na(layx) && !is.na(layy)) layx <- 1
        if (!is.na(layx) && is.na(layy)) layy <- 1
        if (is.na(layx) && is.na(layy))
          layout <- ""
        else
          layout <- paste(", layout=c(", layx, ",", layy, ")", sep="")

        between <- paste(", between=list(x=", betx, ",y=", bety, ")", sep="")
        alternating <- paste("scales=list(",
                             "x=list(alternating=", altx, "), ",
                             "y=list(alternating=", alty, "))", sep="")
        dptb.command <- paste(dptb.command,
                              " ", ## "\n        ",
                              "panel=panel.dotplot.tb",
                              ", factor=", factor.jitter,
                              ", cex=", cex,
                              ", pch=", pch,
                              layout,
                              between,
                              ", ", ## ",\n        ",
                              alternating,
                              ')',
                              sep="")
        doItAndPrint(dptb.command)
        activateMenus()
        tkfocus(CommanderWindow())
        }
    optionsFrame <- tkframe(top)
    buttonsFrame <- tkframe(top)
    OKCancelHelp(helpSubject="panel.dotplot.tb")
    tkgrid(getFrame(factorBox),
           getFrame(responseBox),
           getFrame(groupBox),
           sticky="nw")

    tkgrid(tklabel(scalarsFrame, text=gettextRcmdr("Change defaults if needed:")))
    tkgrid(tklabel(scalarsFrame, text=gettextRcmdr("factor.jitter:")), factor.jitterEntry, sticky="w")
    tkgrid(tklabel(scalarsFrame, text=gettextRcmdr("cex:")), cexEntry, sticky="w")
    tkgrid(tklabel(scalarsFrame, text=gettextRcmdr("pch:")), pchEntry, sticky="w")
    tkgrid(tklabel(scalarsFrame, text=gettextRcmdr("layx:")), layxEntry, sticky="w")
    tkgrid(tklabel(scalarsFrame, text=gettextRcmdr("layy:")), layyEntry, sticky="w")
    tkgrid(tklabel(scalarsFrame, text=gettextRcmdr("betx:")), betxEntry, sticky="w")
    tkgrid(tklabel(scalarsFrame, text=gettextRcmdr("bety:")), betyEntry, sticky="w")
    tkgrid(tklabel(scalarsFrame, text=gettextRcmdr("altx:")), altxEntry, sticky="w")
    tkgrid(tklabel(scalarsFrame, text=gettextRcmdr("alty:")), altyEntry, sticky="w")
    tkgrid(scalarsFrame, sticky="w")
    
    tkgrid(optionsFrame, columnspan=2, sticky="w")
    tkgrid(buttonsFrame, columnspan=2, sticky="w")
    dialogSuffix(rows=3, columns=2)
  }
##source("~/HH-R.package/RcmdrPlugin.HH/R/Dotplot.tb.R")
##source("~/HH-R.package/HH/R/panel.dotplot.tb.R")
FHypothesesPlot <- function(){
    initializeDialog(title=gettextRcmdr("F Distribution"))
    degfree1Var <- tclVar("")
    degfree1Entry <- tkentry(top, width="6", textvariable=degfree1Var)
    degfree2Var <- tclVar("")
    degfree2Entry <- tkentry(top, width="6", textvariable=degfree2Var)
    alphaLeftVar <- tclVar("")
    alphaLeftEntry <- tkentry(top, width="6", textvariable=alphaLeftVar)
    alphaRightVar <- tclVar(".05")
    alphaRightEntry <- tkentry(top, width="6", textvariable=alphaRightVar)
    obsValueVar <- tclVar("")
    obsValueEntry <- tkentry(top, width="6", textvariable=obsValueVar)
    xmaxVar <- tclVar("")
    xmaxEntry <- tkentry(top, width="6", textvariable=xmaxVar)
    ymaxVar <- tclVar("")
    ymaxEntry <- tkentry(top, width="6", textvariable=ymaxVar)
    onOK <- function(){
      closeDialog()
      degfree1 <- as.numeric(tclvalue(degfree1Var))
      degfree2 <- as.numeric(tclvalue(degfree2Var))
      alphaLeft <- as.numeric(tclvalue(alphaLeftVar))
      alphaRight <- as.numeric(tclvalue(alphaRightVar))
      obsValue <- as.numeric(tclvalue(obsValueVar))
      xmax <- as.numeric(tclvalue(xmaxVar))
      ymax <- as.numeric(tclvalue(ymaxVar))
      
      command <- "old.par <- par(oma=c(1,1,0,5))"
      justDoIt(command)
      logger(command)
      
      command.xlim <- if (!is.na(xmax)) 
        paste(", xlim=c(0, ", xmax, ")", sep="")
      else ""
      
      command.ylim <- if (!is.na(ymax)) 
        paste(", ylim=c(0, ", ymax, ")", sep="")
      else ""
      
      command <- "F.setup("
      if (!is.na(degfree1))
        command <- paste(command, "df1=", degfree1, sep="")
      else
        degfree1 <-1
      if (!is.na(degfree2))
        command <- paste(command, ", df2=", degfree2, sep="")
      else
        degfree2 <- Inf

      command <- paste(command,
                       command.xlim,
                       command.ylim,
                       sep="")

      command <- paste(command, ")", sep="")
      justDoIt(command)
      logger(command)

      command.alpha <- ", alpha=c("
      if (!is.na(alphaLeft)) command.alpha <-
        paste(command.alpha, alphaLeft, ", ", sep="")
      command.alpha <-  paste(command.alpha, alphaRight, ")", sep="")

      shade <- if (!is.na(alphaLeft)) "outside" else "right"

      
      command <- paste("F.curve(",
                       "df1=", degfree1,
                       ", df2=", degfree2,
                       command.alpha, 
                       ", shade='", shade,
                       "', col='blue')", sep="")
      doItAndPrint(command)

      ## Observed Value
      if (!is.na(obsValue)) {
        command <- paste("F.observed(", obsValue, ", col='green'",
                       ", df1=", degfree1,
                       ", df2=", degfree2,
                       ")",  sep="")
        doItAndPrint(command)
      }

      command <- "par(old.par)"
      justDoIt(command)
      logger(command)
      
      tkfocus(CommanderWindow())
    }
    OKCancelHelp(helpSubject="dnorm")
    tkgrid(tklabel(top, text=gettextRcmdr("df1 (numerator degrees of freedom)")), degfree1Entry, sticky="e")
    tkgrid(tklabel(top, text=gettextRcmdr("df2 (denominator degrees of freedom)")), degfree2Entry, sticky="e")
    tkgrid(tklabel(top, text=gettextRcmdr("left alpha")), alphaLeftEntry, sticky="e")
    tkgrid(tklabel(top, text=gettextRcmdr("right alpha")), alphaRightEntry, sticky="e")
    tkgrid(tklabel(top, text=gettextRcmdr("Observed Value")), obsValueEntry, sticky="e")
    tkgrid(tklabel(top, text=gettextRcmdr("F max (right-hand side)")), xmaxEntry, sticky="e")
    tkgrid(tklabel(top, text=gettextRcmdr("f(F) max (top side)")), ymaxEntry, sticky="e")
    tkgrid(buttonsFrame, columnspan=1, sticky="w")
    tkgrid.configure(degfree1Entry, sticky="w")
    tkgrid.configure(degfree2Entry, sticky="w")
    tkgrid.configure(alphaLeftEntry, sticky="w")
    tkgrid.configure(alphaRightEntry, sticky="w")
    tkgrid.configure(obsValueEntry, sticky="w")
    tkgrid.configure(xmaxEntry, sticky="w")
    tkgrid.configure(ymaxEntry, sticky="w")
    dialogSuffix(rows=8, columns=1, focus=degfree1Entry)
    }

## source("~/HH-R.package/RcmdrPlugin.HH/R/FHypothesesPlot.R")
## source("~/HH-R.package/HH/R/F.curve.R")
"Interaction2wtRcmdr" <-
function() {
    ## require("HH")
    if (length(grep("HH", search()))==0) stop("Please attach the HH directory.")
    initializeDialog(title=gettextRcmdr("Interaction twoway table"))
    groupBox <- variableListBox(top, Factors(), title=gettextRcmdr("Factors (pick two or more)"), selectmode="multiple")
    responseBox <- variableListBox(top, Numeric(), title=gettextRcmdr("Response Variable (pick one)"))
    onOK <- function() {
        groups <- getSelection(groupBox)
        response <- getSelection(responseBox)
        closeDialog()
        if (2 > length(groups)) {
            errorCondition(recall=Interaction2wtRcmdr, message=gettextRcmdr("Select at least two factors."))
            return()
            }
        if (0 == length(response)) {
            errorCondition(recall=Interaction2wtRcmdr, message=gettextRcmdr("No response variable selected."))
            return()
            }
        .activeDataSet <- ActiveDataSet()
        i2wt.command <- paste("interaction2wt(",
                              response, " ~ ", paste(groups, collapse=' + '),
                              ", data=", .activeDataSet, ')', sep="")
        doItAndPrint(i2wt.command)
        activateMenus()
        tkfocus(CommanderWindow())
        }
    optionsFrame <- tkframe(top)
    buttonsFrame <- tkframe(top)
    OKCancelHelp(helpSubject="interaction2wt")
    tkgrid(getFrame(groupBox), getFrame(responseBox), sticky="nw")
    tkgrid(optionsFrame, columnspan=2, sticky="w")
    tkgrid(buttonsFrame, columnspan=2, sticky="w")
    dialogSuffix(rows=3, columns=2)
  }

MMCmenu <- function() {
    initializeDialog(title=gettextRcmdr("MMC Plot"))
    modelBox <- variableListBox(top, listAOVModels(),
                                title=gettextRcmdr("AOV model (pick one)"))
    checkBoxes(frame="tieFrame", boxes=c("tie"), initialValues=rep(0),
        labels=gettextRcmdr(c("Tiebreaker Plot")))

    onOK <- function() {
      model <- getSelection(modelBox)

      if (length(model) != 1) {
        errorCondition(recall=MMCmenu,
                       message=sprintf(gettextRcmdr("Please select an AOV model.")))
        return()
      }

      if (get(model)$df.residual == 0) {
        errorCondition(recall=MMCmenu,
                       message=sprintf(gettextRcmdr("Model has 0 df for residual.\nPlease pick a different model or cancel the MMC command.")))
        return()
      }

      closeDialog()

      ## doItAndPrint(paste(".activeAOVModel <- '", model, "'", sep=""))
      putRcmdr(".activeAOVModel", model)

      command.factors <-
        paste(model, ".factors <- ", "names(", model, "$xlevels)", sep="")
      eval(parse(text=command.factors))

      command.factors.length <-
        paste("length(", model, ".factors)", sep="")
      focus.and.lmatrows <-  if (eval(parse(text=command.factors.length)) > 1)
        {
        putRcmdr(".MMC2.result", "cancelled") ## to check later
        MMC2menu()
        getRcmdr(".MMC2.result")
      }
      else
        ""
      if (focus.and.lmatrows=="cancelled") return()

      c0a <- paste("right.omd <- .8")
      eval(parse(text=c0a))
      
      command0<- paste("old.omd <- par(omd=c(0, ", right.omd, ", 0,1))", sep="")
      doItAndPrint(command0)

      command1 <- paste(model, ".mmc <- glht.mmc(", model,
                        focus.and.lmatrows, ")", sep="")

      doItAndPrint(command1)

      command2 <- paste(model, ".mmc", sep="")
      doItAndPrint(command2)

      c2a <- paste(command2, "$mca$table[,'estimate']", sep="")
      c2b <- paste("x.left <- -max(abs(",
                   c2a, "))",
                   sep="")
      eval(parse(text=c2b))

      c2c <- paste("x.right <- max(", command2,
                   "$mca$table[,'upper'])",
                   sep="")
      eval(parse(text=c2c))

      eval(parse(text=(paste("x.offset <- (x.left+x.right)/2"))))


      c2e <- paste("y.range <- range(", command2,
                   "$none$table[,'estimate'])",
                   sep="")
      eval(parse(text=c2e))

      eval(parse(text=("ry <- mean(y.range) + (y.range - mean(y.range))*(-2*x.left+x.offset)/(-2*x.left)*1.4")))

      command3 <- paste("plot(", model, ".mmc, x.offset=",
                        x.offset, ", ry=", deparse(ry), ")", sep="")
      doItAndPrint(command3)

      if (tclvalue(tieVariable)=="1") {
        command4 <-
          paste("plot.matchMMC(", model, ".mmc$mca, xlabel.print=FALSE)",
                sep="")
        doItAndPrint(command4)
      }

      command5<- "par(old.omd)"
      doItAndPrint(command5)

      
      logger("## The default placement of MMC plots may not be ideal.
## You may need to adjust the x.offset and/or ry arguments.
## You may need to adjust the par('omd') parameter before the plot.
## See the help file for MMC for examples.")
    }

    OKCancelHelp(helpSubject="MMC")
    tkgrid(getFrame(modelBox), sticky="w")
    tkgrid(tieFrame, sticky="w")
    tkgrid(buttonsFrame, columnspan=2, sticky="w")
    dialogSuffix(rows=3, columns=2)
  }

MMC2menu <- function() {
    model <- getRcmdr(".activeAOVModel")
    initializeDialog(title=gettextRcmdr(paste("MMC Plot for", model)))
    ## checkBoxes(frame="tieFrame", boxes=c("tie"), initialValues=rep(0),
    ##     labels=gettextRcmdr(c("Tiebreaker Plot")))

    xlevels <- eval(parse(text=paste(model, "$xlevels", sep="")))
    factors <- names(xlevels)
    focusBox <-
      variableListBox(top,
                      factors,
                      title=gettextRcmdr("focus factor (pick one)"))

    tmp <- eval(parse(text=paste("glht(get(model), linfct=mcp(",
                        factors[1], "='Tukey'))", sep="")))
    lmat.rownames <- dimnames(tmp$linfct)[[2]]

    checkBoxes(frame="choose.lmat.rowsFrame", boxes=c("lmatrows"),
               initialValues=rep(1),
        labels=gettextRcmdr(c("Accept the default lmat.rows.")))

    lmat.rowsBox <- variableListBox(top,
                                lmat.rownames,
                                title=gettextRcmdr("Or select all the lmat rows corresponding to the focus factor.")
                                    , selectmode="multiple")

    onOK <- function() {

      focus <- getSelection(focusBox)
        if (length(focus) != 1) {
            errorCondition(recall=MMC2menu,
                           message=sprintf(gettextRcmdr("Please select a focus factor.")))
            return()
            }

      if (tclvalue(lmatrowsVariable)=="1") {
        lmat.rows <- grep(paste("^", focus, sep=""),
                          gsub(".*:.*", "", lmat.rownames))

        logger(paste("Default lmat.rows:",
                     paste(lmat.rownames[lmat.rows], collapse=" ")))
        if (length(lmat.rows)==0) {
          logger("We can't verify that the rows you selected are correct.")
          errorCondition(recall=MMC2menu,
                         message=sprintf(gettextRcmdr("Please select the lmat rowfrom the list below.")))
        }
      }
      else {
        lmat.rowselected <- getSelection(lmat.rowsBox)
        lmat.rows <- match(lmat.rowselected, lmat.rownames)

        if (!((length(xlevels[[focus]]) == length(lmat.rows)+1) &&
              all(substring(lmat.rowselected, 1, nchar(focus)) ==  focus) &&
              !any(grep("\\[", sub("\\[", "\\*", lmat.rownames)) %in% lmat.rows) &&
              all(diff(lmat.rows)==1))) {
          errorCondition(recall=MMC2menu,
                         message=sprintf(gettextRcmdr("Please select the lmat rows corresponding to the focus factor.")))
          return()
        }
      }
      
      putRcmdr(".MMC2.result",
               paste(", focus='", focus,
                     "', lmat.rows=", deparse(lmat.rows),
                     sep=""))
      closeDialog()


    }

    OKCancelHelp(helpSubject="MMC")
    tkgrid(getFrame(focusBox), sticky="w")
    tkgrid(choose.lmat.rowsFrame, sticky="w")
    tkgrid(getFrame(lmat.rowsBox), sticky="w")
    tkgrid(buttonsFrame, columnspan=2, sticky="w")
    dialogSuffix(rows=3, columns=2)
}

AOVModelsP <-
  function (n = 1) 
  length(listAOVModels()) >= n


## source("~/HH-R.package/RcmdrPlugin.HH/R/MMC.R")
"PredictModel" <-
function(){
    require("abind")
    env <- environment()
    initializeDialog(title=gettextRcmdr("Enter X data for new observations"))
    confLevelVar <- tclVar(".95")
    confLevelEntry <- tkentry(top, width="6", textvariable=confLevelVar)
    checkBoxes(frame="seFrame", boxes=c("se"), initialValues=rep(0,1),
        labels=gettextRcmdr(c("Standard Error")))
    outerTableFrame <- tkframe(top)
    assign(".tableFrame", tkframe(outerTableFrame), envir=env)
    setUpTable <- function(...) {
        tkdestroy(get(".tableFrame", envir=env))
        assign(".tableFrame", tkframe(outerTableFrame), envir=env)
        nrows <- as.numeric(tclvalue(rowsValue))
        ncols <- length(colsNames)
        make.col.names <- "tklabel(.tableFrame, text='')"
        for (j in seq(length.out=ncols)) {
            col.varname <- colsNames[j] ## paste(".colname.", j, sep="")
            assign(col.varname, tclVar(colsNames[j]), envir=env)
            make.col.names <- paste(make.col.names, ", ", "tkentry(.tableFrame, width='12', textvariable=", 
                    col.varname, ")", sep="")
            }
        if (ncols > 0) {
eval(parse(text=paste("tkgrid(", make.col.names, ")", sep="")), envir=env)
        for (i in 1:nrows) {   
            varname <- paste(".tab.", i, ".1", sep="") 
            assign(varname, tclVar("") , envir=env)
            row.varname <- paste(".rowname.", i, sep="")
            assign(row.varname, tclVar(i), envir=env)
            make.row <- paste("tkentry(.tableFrame, width='12', textvariable=",
                row.varname, ")", sep="")
            make.row <- paste(make.row, ", ", "tkentry(.tableFrame, width='12', textvariable=", 
                varname, ")", sep="")
            if (ncols > 1)
            for (j in 2:ncols) {
                varname <- paste(".tab.", i, ".", j, sep="")
                assign(varname, tclVar(""), envir=env)
                make.row <- paste(make.row, ", ", "tkentry(.tableFrame, width='12', textvariable=", 
                    varname, ")", sep="")
## recover()
                }
            eval(parse(text=paste("tkgrid(", make.row, ")", sep="")), envir=env)
            }
        tkgrid(get(".tableFrame", envir=env), sticky="w")
      }
      }

    .active.model <- ActiveModel()

    colsNames <- if (is.null(.active.model)) NULL else dimnames(attr(get(.active.model)$terms,"factors"))[[2]]

    rowColFrame <- tkframe(top)
    rowsValue <- tclVar("1")
    rowsSlider <- tkscale(rowColFrame, from=1, to=10, showvalue=FALSE, variable=rowsValue,
        resolution=1, orient="horizontal", command=setUpTable)
    rowsShow <- tklabel(rowColFrame, textvariable=rowsValue, width=2, justify="right")

    onOK <- function() {
    if (is.null(.active.model)) {
      errorCondition(recall=PredictModel, message=sprintf(gettextRcmdr("No active model.  Please press Cancel.")))
      return()
    }
        nrows <- as.numeric(tclvalue(rowsValue))
        ncols <- length(colsNames)

    col.names <- dimnames(attr(get(.active.model)$terms,"factors"))[[2]]
    newdata <- get(activeDataSet())[1:nrows, col.names, drop=FALSE]
    for (i in names(newdata)) newdata[[i]][] <- NA
    ## sapply(newdata, levels)
    row.names <- rep("", nrows)
    for (i in 1:nrows) row.names[i] <- 
      eval(parse(text=paste("tclvalue(", paste(".rowname.", i, sep=""),")", sep="")))
        for (i in 1:nrows){
            for (j in 1:ncols){
                varname <- paste(".tab.", i, ".", j, sep="")
                cell.value <-
                  eval(parse(text=paste("tclvalue(", varname,")", sep="")))
                aDSj <- get(activeDataSet())[[col.names[j]]]
                if (is.factor(aDSj) || is.character(aDSj))
                  newdata[i,j] <- cell.value
                else
                  newdata[i,j] <- as.numeric(cell.value)
                }
            }
        newdata <- na.omit(newdata)
        if (prod(dim(newdata)) != nrows*ncols) {
            errorCondition(recall=PredictModel,
                           message=sprintf(gettextRcmdr("Number of valid entries (%d)\nnot equal to number of rows (%d) * number of columns (%d)."),
                             prod(dim(newdata)), nrows, ncols))
            return()
            }
        if (length(unique(row.names)) != nrows) {
            errorCondition(recall=PredictModel, message=gettextRcmdr("Row names are not unique."))
            return()
            }     
        if (length(unique(col.names)) != ncols) {
            errorCondition(recall=PredictModel, message=gettextRcmdr("Column names are not unique."))
            return()
            }     
    closeDialog()

    confLevel <- as.numeric(tclvalue(confLevelVar))
    j <- 1
    command <- paste(".NewData <- data.frame(", col.names[j], "=",
                     deparse(newdata[,j]), sep="")
    if (ncols > 1)
      for (j in 2:ncols) {
        command <- paste(command, ", ", col.names[j], "=",
                         deparse(newdata[,j]), sep="")
      }
    command <- paste(command, ", ", "row.names=",
                     deparse(row.names),")", sep="")
    doItAndPrint(command)
    doItAndPrint(".NewData  # Newdata")

    command <- paste('predict(',
                     .active.model,
                     ', newdata=.NewData, interval="',
                     tclvalue(predictVariable),
                     '", level=', tclvalue(confLevelVar),
                     ', se.fit=', ("1"==tclvalue(seVariable)) ,
                     ')', sep="")
    doItAndPrint(command)
    ## logger("remove(.NewData)") 
    ## remove(.NewData, envir=.GlobalEnv)                                                      
    tkfocus(CommanderWindow())
  }

    OKCancelHelp(helpSubject="predict")
    
    radioButtons(name="predict",
                 buttons=c("none",
                   "confidence",
                   "prediction"),
                 values=c("none", "confidence", "prediction"),
                 initialValue="confidence",
                 labels=gettextRcmdr(c
                   ("point estimate only",
                    "confidence interval for mean",
                    "prediction interval for individual")),
                 title=gettextRcmdr("Prediction Interval"))
    
    if (length(colsNames) > 0) {
    
    tkgrid(tklabel(rowColFrame,
                   text=gettextRcmdr("Number of Rows:")),
           rowsSlider, rowsShow, sticky="w")
    tkgrid(rowColFrame, sticky="w")
    tkgrid(tklabel(top,
                   text=gettextRcmdr("Enter X values:"),
                   fg="blue"), sticky="w")
    tkgrid(outerTableFrame, sticky="w")
    tkgrid(predictFrame, sticky="w")
    tkgrid(tklabel(top, text=gettextRcmdr("Confidence Level")),
           confLevelEntry, sticky="w")
    tkgrid(seFrame, sticky="w")
  }
    tkgrid(buttonsFrame, columnspan=2, sticky="w")
    dialogSuffix(rows=7, columns=2)
    }

## dimnames(attr(RegModel.1$terms,"factors"))[[2]]


## source("~/HH-R.package/RcmdrPlugin.HH/R/PredictModel.R")
Projector <- function() {
  Rcmdr <- options()$Rcmdr
  Projector.options <-
    list(log.font.size = 15,
         log.width = 54,
         log.height = 6, 
         output.height = 18,
         scale.factor = 1.4)
  Rcmdr[names(Projector.options)] <- Projector.options
  options(Rcmdr=Rcmdr)
  trellis.par.set(list(superpose.symbol=list(
                         pch=rep(16,
                           length(trellis.par.get("superpose.symbol")$pch))),
                       plot.symbol=list(pch=16)))
  par(pch=16)
  
  putRcmdr("autoRestart", TRUE)
  closeCommander(ask=FALSE)
  Commander()
}

## source("~/HH-R.package/RcmdrPlugin.HH/R/Projector.R")
"QQPlot.HH" <-
function()
  ## this function modified by Martin Maechler
  ## this function modified by Richard M. Heiberger to add Shapiro-Wilk test
{
    require("car")
    initializeDialog(title=gettextRcmdr("Quantile-Comparison (QQ) Plot"))
    xBox <- variableListBox(top, Numeric(), title=gettextRcmdr("Variable (pick one)"))
    onOK <- function(){
        x <- getSelection(xBox)
        closeDialog()
       if (0 == length(x)) {
            errorCondition(recall=QQPlot.HH, message=gettextRcmdr("You must select a variable.")) 
            return()
            }
        dist <- tclvalue(distVariable)
        save <- options(warn=-1)
        on.exit(options(save))
        retryMe <- function(msg) {
            Message(message= msg, type="error")
            QQPlot.HH()
        }
        switch(dist,
               "norm" = {
                 args <- 'dist= "norm"'
                 ShapWilk <- as.numeric(tclvalue(ShapWilkVariable))
               },
               "t" =  {
                   df <- tclvalue(tDfVariable)
                   df.num <- as.numeric(df)
                   if (is.na(df.num) || df.num < 1) {
                       retryMe(gettextRcmdr("df for t must be a positive number."))
                       return()
                   }
                   args <- paste('dist="t", df=', df, sep="")
               },
               "chisq" = {
                   df <- tclvalue(chisqDfVariable)
                   df.num <- as.numeric(df)
                   if (is.na(df.num) || df.num < 1) {
                       retryMe(gettextRcmdr("df for chi-square must be a positive number."))
                       return()
                   }
                   args <- paste('dist="chisq", df=', df, sep="")
               },
               "f" = {
                   df1 <- tclvalue(FDf1Variable)
                   df2 <- tclvalue(FDf2Variable)
                   df.num1 <- as.numeric(df1)
                   df.num2 <- as.numeric(df2)
                   if (is.na(df.num1) || df.num1 < 1 ||
                       is.na(df.num2) || df.num2 < 1) {
                       retryMe(gettextRcmdr("numerator and denominator \ndf for F must be positive numbers."))
                       return()                            
                   }
                   args <- paste('dist="f", df1=', df1, ', df2=', df2, sep="")
               },
               ## else -- other `dist' :
           {
               dist <- tclvalue(otherNameVariable)
               params <- tclvalue(otherParamsVariable)
               args <- paste('dist="', dist,'", ', params, sep="")
           }) # end{switch}
        .activeDataSet <- ActiveDataSet()
        labels <-
            if ("1" == tclvalue(identifyVariable))
                paste("rownames(", .activeDataSet, ")", sep="")
            else "FALSE"
        command <- paste("qq.plot", "(", .activeDataSet, "$", x, ", ", args,
                          ", labels=", labels, ")", sep="")
        command2 <- paste("shapiro.test", "(", .activeDataSet, "$", x, ")", sep="")
        doItAndPrint(command)
        if (dist=="norm" && ShapWilk) doItAndPrint(command2)
        activateMenus()
        tkfocus(CommanderWindow())
    }
    OKCancelHelp(helpSubject="qq.plot")
    distFrame <- tkframe(top)
    distVariable <- tclVar("norm")
    normalButton <- tkradiobutton(distFrame, variable=distVariable, value="norm")
    tButton <- tkradiobutton(distFrame, variable=distVariable, value="t")
    chisqButton <- tkradiobutton(distFrame, variable=distVariable, value="chisq")
    FButton <- tkradiobutton(distFrame, variable=distVariable, value="f")
    otherButton <- tkradiobutton(distFrame, variable=distVariable, value="other")
    checkBoxes(frame="normFrame", boxes=c("ShapWilk"), initialValues=c("1"),
        labels=gettextRcmdr(c("Shapiro-Wilk test of normality")))
    tDfFrame <- tkframe(distFrame)
    tDfVariable <- tclVar("")
    tDfField <- tkentry(tDfFrame, width="6", textvariable=tDfVariable)
    chisqDfFrame <- tkframe(distFrame)
    chisqDfVariable <- tclVar("")
    chisqDfField <- tkentry(chisqDfFrame, width="6", textvariable=chisqDfVariable)
    FDfFrame <- tkframe(distFrame)
    FDf1Variable <- tclVar("")
    FDf1Field <- tkentry(FDfFrame, width="6", textvariable=FDf1Variable)
    FDf2Variable <- tclVar("")
    FDf2Field <- tkentry(FDfFrame, width="6", textvariable=FDf2Variable)
    otherParamsFrame <- tkframe(distFrame)
    otherParamsVariable <- tclVar("")
    otherParamsField <- tkentry(otherParamsFrame, width="30", textvariable=otherParamsVariable)
    otherNameVariable <- tclVar("")
    otherNameField <- tkentry(otherParamsFrame, width="10", textvariable=otherNameVariable)
    identifyVariable <- tclVar("0")
    identifyFrame <- tkframe(top)
    identifyCheckBox <- tkcheckbutton(identifyFrame, variable=identifyVariable)
    tkgrid(getFrame(xBox), sticky="nw")
    tkgrid(tklabel(identifyFrame, text=gettextRcmdr("Identify observations with mouse")),
           identifyCheckBox, sticky="w")
    tkgrid(identifyFrame, sticky="w")
    tkgrid(tklabel(distFrame, text=gettextRcmdr("Distribution"), fg="blue"), columnspan=6, sticky="w")
    tkgrid(tklabel(distFrame, text=gettextRcmdr("Normal")), normalButton, normFrame, sticky="w")
    tkgrid(tklabel(tDfFrame, text=gettextRcmdr("df = ")), tDfField, sticky="w")
    tkgrid(tklabel(distFrame, text="t"), tButton, tDfFrame, sticky="w")
    tkgrid(tklabel(chisqDfFrame, text=gettextRcmdr("df = ")), chisqDfField, sticky="w")
    tkgrid(tklabel(distFrame, text=gettextRcmdr("Chi-square")), chisqButton,
           chisqDfFrame, sticky="w")
    tkgrid(tklabel(FDfFrame, text=gettextRcmdr("Numerator df = ")), FDf1Field,
           tklabel(FDfFrame, text=gettextRcmdr("Denominator df = ")), FDf2Field, sticky="w")
    tkgrid(tklabel(distFrame, text="F"), FButton, FDfFrame, sticky="w")
    tkgrid(tklabel(otherParamsFrame, text=gettextRcmdr("Specify: ")),
           otherNameField, tklabel(otherParamsFrame, text=gettextRcmdr("Parameters: ")),
           otherParamsField, sticky="w")
    tkgrid(tklabel(distFrame, text=gettextRcmdr("Other")), otherButton,
           otherParamsFrame, sticky="w")
    tkgrid(distFrame, sticky="w")
    tkgrid(buttonsFrame, sticky="w")
    dialogSuffix(rows=5, columns=1)
    }

R_options <- function() {

  initializeDialog(title=gettextRcmdr("R Options"))

  parFrame <- tkframe(top) 
  pchVar <- tclVar(gettextRcmdr("16"))
  pchEntry <- tkentry(parFrame, width=3, textvariable=pchVar)      

  onOK <- function(){

    pch <- tclvalue(pchVar)
    if ("" == pch) pch="16"
    pch <- trim.blanks(pch)

    old.warn <- options(warn=-1)
    pchValue <- as.numeric(pch)
    if (is.na(pchValue)) {
      if (nchar(pch) > 1) pchValue <- substring(pch,1,1)
      pchValue <- paste('"', pchValue, '"', sep="")
    }
    else
      pchValue <- pch
    options(old.warn)

    command <- paste('trellis.par.set(list(
                 superpose.symbol=list(pch=rep(', pchValue, ', length(trellis.par.get("superpose.symbol")$pch))),
                 plot.symbol=list(pch=', pchValue, ')))', sep='')
    doItAndPrint(command)
    
    command <- paste('par(pch=',pchValue,')', sep='')
    doItAndPrint(command)

 
    stars <- (tclvalue(starsVariable) ==  "1")
    command <- paste('options(show.signif.stars=', stars, ')')
    doItAndPrint(command)
    closeDialog()
    tkfocus(CommanderWindow())
  }
  OKCancelHelp(helpSubject="options")
 
  checkBoxes(frame="starsFrame", boxes=c("stars"), initialValues=c("1"),
             labels=gettextRcmdr(c("Show significance stars")))
  
  tkgrid(tklabel(parFrame, text=gettextRcmdr("Plotting characters")), pchEntry, stick="w")
  tkgrid(parFrame, sticky="w")
  tkgrid(starsFrame, sticky="w")
  tkgrid(buttonsFrame, sticky="w")
  dialogSuffix(rows=2, columns=1)
}

## source("~/HH-R.package/RcmdrPlugin.HH/R/R_options.R")
Regr1Plot <- function() {
    require("graphics")
    initializeDialog(title=gettextRcmdr("Squared Residuals"))
    .numeric <- Numeric()
    variablesFrame <- tkframe(top)
    xBox <- variableListBox(variablesFrame, .numeric, title=gettextRcmdr("x-variable (pick one)"))
    yBox <- variableListBox(variablesFrame, .numeric, title=gettextRcmdr("y-variable (pick one)"))
    gBox <- variableListBox(variablesFrame, Factors(),
                            title=gettextRcmdr("group-variable (pick zero or one)"),
                            selectmode="single", initialSelection=-1)
    workingFrame <- tkframe(top)
    woFrame <- tkframe(workingFrame)
    checkBoxes(window=woFrame,
               frame="optionsFrame",
               boxes=c("jitterX", "points.yhat"),
               initialValues=c(0, 1),
               labels=gettextRcmdr(c("Jitter x-variable", "Display Y.hat points")))
    radioButtons(window=workingFrame,
                 name="residuals",
                 buttons=c("square",
                   "line",
                   "none"),
                 values=c("'square'", "'line'", "FALSE"),
                 initialValue="'square'",
                 labels=gettextRcmdr(c
                   ("residuals---squares",
                    "residuals---straight line",
                    "point estimate only")),
                 title=gettextRcmdr("Residual Display"))
    radioButtons(window=workingFrame,
                 name="model",
                 buttons=c("linear", "active"),
                 values=c("0", "1"),
                 initialValue="0",
                 labels=gettextRcmdr(c
                   ("default linear model",
                    "Active model")),
                 title=gettextRcmdr("Model"))
    subsetBox()    
    labelsFrame <- tkframe(top)
    xlabVar <- tclVar(gettextRcmdr("<auto>"))
    ylabVar <- tclVar(gettextRcmdr("<auto>"))
    xlabFrame <- tkframe(labelsFrame)
    xlabEntry <- tkentry(xlabFrame, width="25", textvariable=xlabVar)
    xlabScroll <- tkscrollbar(xlabFrame, orient="horizontal",
        repeatinterval=5, command=function(...) tkxview(xlabEntry, ...))
    tkconfigure(xlabEntry, xscrollcommand=function(...) tkset(xlabScroll, ...))
    tkgrid(tklabel(xlabFrame, text=gettextRcmdr("x-axis label"), fg="blue"), sticky="w")
    tkgrid(xlabEntry, sticky="w")
    tkgrid(xlabScroll, sticky="ew")
    ylabFrame <- tkframe(labelsFrame)
    ylabEntry <- tkentry(ylabFrame, width="25", textvariable=ylabVar)
    ylabScroll <- tkscrollbar(ylabFrame, orient="horizontal",
        repeatinterval=5, command=function(...) tkxview(ylabEntry, ...))
    tkconfigure(ylabEntry, xscrollcommand=function(...) tkset(ylabScroll, ...))
    tkgrid(tklabel(ylabFrame, text=gettextRcmdr("y-axis label"), fg="blue"), sticky="w")
    tkgrid(ylabEntry, sticky="w")
    tkgrid(ylabScroll, sticky="ew")
    tkgrid(xlabFrame, tklabel(labelsFrame, text="     "), ylabFrame, sticky="w")    
    parFrame <- tkframe(top) 


    limitsFrame <- tkframe(top)
    xminVar <- tclVar("")
    xminEntry <- tkentry(limitsFrame, width="6", textvariable=xminVar)
    xmaxVar <- tclVar("")
    xmaxEntry <- tkentry(limitsFrame, width="6", textvariable=xmaxVar)
    yminVar <- tclVar("")
    yminEntry <- tkentry(limitsFrame, width="6", textvariable=yminVar)
    ymaxVar <- tclVar("")
    ymaxEntry <- tkentry(limitsFrame, width="6", textvariable=ymaxVar)
   
    onOK <- function(){
        x <- getSelection(xBox)
        y <- getSelection(yBox)
        g <- getSelection(gBox)

        xmin <- as.numeric(tclvalue(xminVar))
        xmax <- as.numeric(tclvalue(xmaxVar))
        ymin <- as.numeric(tclvalue(yminVar))
        ymax <- as.numeric(tclvalue(ymaxVar))

        number.xna <- is.na(xmin) + is.na(xmax)
        if (number.xna==1) {
            errorCondition(recall=Regr1Plot,
                           message=gettextRcmdr("Both or neither xlim values must be numbers."))
            return()
          }
        xlim.command <- ifelse(number.xna==0, deparse(c(xmin, xmax)), "")

        number.yna <- is.na(ymin) + is.na(ymax)
        if (number.yna==1) {
            errorCondition(recall=Regr1Plot,
                           message=gettextRcmdr("Both or neither ylim values must be numbers."))
            return()
          }
         ylim.command <- ifelse(number.yna==0, deparse(c(ymin, ymax)), "")



        closeDialog()
        if (length(x) == 0 || length(y) == 0){
            errorCondition(recall=Regr1Plot, message=gettextRcmdr("You must select two variables"))
            return()
            }
        if (x == y) {
            errorCondition(recall=Regr1Plot, message=gettextRcmdr("x and y variables must be different"))
            return()
            }
        .activeDataSet <- ActiveDataSet()
        Dx <- paste(.activeDataSet, "$", x, sep="")
        Dy <- paste(.activeDataSet, "$", y, sep="")
        Dg <- paste(.activeDataSet, "$", g, sep="")
        if ("1" == tclvalue(jitterXVariable)) Dx <- paste("jitter(", Dx, ")", sep="")
        subset <- tclvalue(subsetVariable)
        subset <- if (trim.blanks(subset) == gettextRcmdr("<all valid cases>")) "" 
            else paste(", subset=", subset, sep="")

        points.yhat <- ("1" == tclvalue(points.yhatVariable))
        points.yhat.command <- ifelse(points.yhat, "", ", points.yhat=FALSE")

        xlab <- trim.blanks(tclvalue(xlabVar))
        if(xlab == gettextRcmdr("<auto>")) xlab <- x
        xlab <- paste(', xlab="', xlab, '"', sep="")
        ylab <- trim.blanks(tclvalue(ylabVar))
        if(ylab == gettextRcmdr("<auto>")) ylab <- y
        ylab <- paste(', ylab="', ylab, '"', sep="")

        ADS.x <- get(.activeDataSet)[[x]]
        ADS.y <- get(.activeDataSet)[[y]]
        ADS.lm <- lm(ADS.y ~ ADS.x)
        ADS.ylim <- range(ADS.y, predict(ADS.lm))

        aspect.x.y <- diff(range(ADS.x)) / diff(range(ADS.y))
        ADS.xlim <- range(ADS.x, ADS.x + resid(ADS.lm)*aspect.x.y)
        
        if(xlim.command == "") xlim.command <- deparse(ADS.xlim)
        xlim <- paste(', xlim=', xlim.command, sep="")
        if(ylim.command == "") ylim.command <- deparse(ADS.ylim)
        ylim <- paste(', ylim=', ylim.command, sep="")

        model <- if (0==tclvalue(modelVariable)) ""
        else {
          .activeModel <- ActiveModel()
          if (is.null(.activeModel)) {
            errorCondition(recall=Regr1Plot,
                           message=gettextRcmdr("No Active model"))
            return()
          }
          paste(', model=', .activeModel, sep="")
        }
        main <- if (model == "")
          paste('Residuals from model: ', y, ' ~ ', x)
        else
          paste('Residuals from model: ', deparse(as.formula(get(ActiveModel()))))
        
        if (length(g) == 0) {
          doItAndPrint(paste("regr1.plot(", "x=", Dx, ", y=", Dy,
                             ", resid.plot=", tclvalue(residualsVariable),
                             xlab, ylab, ", cex=1.3",
                             xlim, ylim, model,
                             subset,
                             points.yhat.command,
                             ", main='", main, "')", sep=""))
        }
        else {
          doItAndPrint(paste("regr1.plot(", "x=", Dx, ", y=", Dy,
                             ", resid.plot=", tclvalue(residualsVariable),
                             ", col=", "match(", Dg, ", levels(", Dg, "))+1",
                             xlab, ylab, ", cex=1.3",
                             xlim, ylim, model,
                             subset,
                             points.yhat.command,
                             ", main='", main, "')", sep=""))
        }
        activateMenus()
        tkfocus(CommanderWindow())
        }
    OKCancelHelp(helpSubject="regr1.plot")
    tkgrid(getFrame(xBox), getFrame(yBox),
           getFrame(gBox), columnspan=1, sticky="nw") 
    tkgrid(variablesFrame, sticky="w")   
    tkgrid(subsetFrame, sticky="w")

    tkgrid(tklabel(woFrame, text=gettextRcmdr("Options"), fg="blue"), sticky="w")
    tkgrid(optionsFrame, sticky="w")
    tkgrid(modelFrame, residualsFrame,
           woFrame, columnspan=1, sticky="nw") 
    tkgrid(workingFrame, sticky="w")   
    
    limitsnamesFrame <- tkframe(top)
    tkgrid(tklabel(limitsnamesFrame,
                   text=gettextRcmdr("xlim and ylim (try defaults first)"),
                   fg="blue"), sticky="w")
    tkgrid(tklabel(limitsnamesFrame,
                   text="xmin       xmax       ymin       ymax"), sticky="w")
    tkgrid(limitsnamesFrame, sticky="w")
    tkgrid(xminEntry, xmaxEntry, yminEntry, ymaxEntry)
    tkgrid(limitsFrame, sticky="w")
    tkgrid(labelsFrame, sticky="w")
    tkgrid(tklabel(top, text=" "))    
    tkgrid(buttonsFrame, columnspan=2, sticky="w")
    dialogSuffix(rows=8, columns=2)
    }

## source("~/HH-R.package/RcmdrPlugin.HH/R/Regr1Plot.R")
"Scatter3DDialog.HH" <-
function(){
    use.rgl <- options("Rcmdr")[[1]]$use.rgl
    if (length(use.rgl) == 0 || use.rgl) require(rgl)    
    initializeDialog(title=gettextRcmdr("3D Scatterplot"))
    variablesFrame <- tkframe(top)
    .numeric <- Numeric()
    xBox <- variableListBox(variablesFrame, .numeric, title=gettextRcmdr("Explanatory variables (pick two)"), selectmode="multiple",
        initialSelection=NULL, listHeight=7)
    yBox <- variableListBox(variablesFrame, .numeric, title=gettextRcmdr("Response variable (pick one)"), listHeight=7)
    surfacesFrame <- tkframe(top)
    identifyPoints <- tclVar("0") 
    identifyPointsCheckBox <- tkcheckbutton(surfacesFrame, variable=identifyPoints)
    gridLines <- tclVar("1")
    gridLinesCheckBox <- tkcheckbutton(surfacesFrame, variable=gridLines)
    rglOpen <- tclVar("0")
    rglOpenCheckBox <- tkcheckbutton(surfacesFrame, variable=rglOpen)
    linearLSSurface <- tclVar("1")
    linearLSCheckBox <- tkcheckbutton(surfacesFrame, variable=linearLSSurface)
    betaMultiplierVariable <- tclVar(gettextRcmdr("1.0"))
    betaMultiplierField <- tkentry(surfacesFrame, width="6", textvariable=betaMultiplierVariable)
    quadLSSurface <- tclVar("0")
    quadLSCheckBox <- tkcheckbutton(surfacesFrame, variable=quadLSSurface)
    nonparSurface <- tclVar("0")
    nonparCheckBox <- tkcheckbutton(surfacesFrame, variable=nonparSurface)
    dfNonparVariable <- tclVar(gettextRcmdr("<auto>"))
    dfNonparField <- tkentry(surfacesFrame, width="6", textvariable=dfNonparVariable)
    additiveSurface <- tclVar("0")
    additiveCheckBox <- tkcheckbutton(surfacesFrame, variable=additiveSurface)
    dfAddVariable <- tclVar(gettextRcmdr("<auto>"))
    dfAddField <- tkentry(surfacesFrame, width="6", textvariable=dfAddVariable)
    bgFrame <- tkframe(top)
    bgVariable <- tclVar("white")
    whiteButton <- tkradiobutton(bgFrame, variable=bgVariable, value="white")
    blackButton <- tkradiobutton(bgFrame, variable=bgVariable, value="black")
    residFrame <- tkframe(top)
    residVariable <- tclVar("line")
    lineButton <- tkradiobutton(residFrame, variable=residVariable, value="line")
    squareButton <- tkradiobutton(residFrame, variable=residVariable, value="square")
    onOK <- function(){
        x <- getSelection(xBox)
        y <- getSelection(yBox)
        closeDialog()
        if (length(y) == 0) {
            errorCondition(recall=Scatter3DDialog.HH, message=gettextRcmdr("You must select a response variable."))
            return()
            }
        if (2 != length(x)) {
            errorCondition(recall=Scatter3DDialog.HH, message=gettextRcmdr("You must select 2 explanatory variables."))
            return()
            }
        if (is.element(y, x)) {
            errorCondition(recall=Scatter3DDialog.HH, message=gettextRcmdr("Response and explanatory variables must be different."))
            return()
            }
        grid <- if (tclvalue(gridLines) == 1) "TRUE" else "FALSE"
        lin <- if(tclvalue(linearLSSurface) == 1) '"linear"'
        quad <- if(tclvalue(quadLSSurface) == 1) '"quadratic"'
        nonpar <- if (tclvalue(nonparSurface) == 1) '"smooth"'
        additive <- if (tclvalue(additiveSurface) == 1) '"additive"'
        surfaces <- c(lin, quad, nonpar, additive)
        nsurfaces <- length(surfaces)
        opts <- options(warn=-1)
        betaMultiplier <- tclvalue(betaMultiplierVariable)
        betaMultiplier <- if (betaMultiplier == gettextRcmdr("1.0")) "" else paste(", coef.ratio=", as.numeric(betaMultiplier), sep="")
        dfNonpar <- tclvalue(dfNonparVariable)
        dfNonpar <- if (dfNonpar == gettextRcmdr("<auto>")) "" else paste(", df.smooth=", as.numeric(dfNonpar), sep="")
        dfAdd <- tclvalue(dfAddVariable)
        dfAdd <- if (dfAdd == gettextRcmdr("<auto>")) "" else paste(", df.additive=", as.numeric(dfAdd), sep="")
        options(opts)
        fit <- if (nsurfaces == 0) ", surface=FALSE"
            else if (nsurfaces == 1) paste(", fit=", surfaces, sep="")
            else paste(", fit=c(", paste(surfaces, collapse=","), ")", sep="")
        bg <- tclvalue(bgVariable)
        resid <- tclvalue(residVariable)
        .activeDataSet <- ActiveDataSet()
        if (.groups != FALSE){ 
            groups <- paste(", groups=", .activeDataSet, "$", .groups, sep="")
            parallel <- paste(", parallel=", .linesByGroup, sep="")
            }
        else groups <- parallel <- ""
        if (tclvalue(rglOpen) == 1) doItAndPrint("rgl.open()")
        command <- paste("scatter3d.HH(", .activeDataSet, "$", x[1], ", ", 
            .activeDataSet, "$", y, ", ", .activeDataSet, "$", x[2], fit, dfNonpar, 
            dfAdd, groups, parallel, ', bg="', bg, '", grid=', grid,
                         ', squares=', resid=="square", betaMultiplier,
            ', xlab="', x[1], '", ylab="', y, '", zlab="', x[2], '")', sep="")
        doItAndPrint(command)
        putRcmdr("rgl", TRUE)
        command <- paste("identify3d(", .activeDataSet, "$", x[1], ", ", 
            .activeDataSet, "$", y, ", ", .activeDataSet, "$", x[2], groups,
            ", labels=row.names(", .activeDataSet, "))", sep="")
        putRcmdr("Identify3d", command)
        .Tcl("update")
        if (tclvalue(identifyPoints) == 1) doItAndPrint(command)
        activateMenus()
        tkfocus(CommanderWindow())
        rgl.bringtotop()
        }
    groupsBox(Scatter3DDialog.HH, plotLinesByGroup=TRUE, plotLinesByGroupsText=gettextRcmdr("Parallel regression surfaces"))
    OKCancelHelp(helpSubject="Scatter3DDialog")
    tkgrid(getFrame(yBox), tklabel(variablesFrame, text="  "), getFrame(xBox), sticky="nw")
    tkgrid(variablesFrame, sticky="nw")
    tkgrid(tklabel(surfacesFrame, text=gettextRcmdr("Identify observations\nwith mouse")), identifyPointsCheckBox, sticky="w")
    tkgrid(tklabel(surfacesFrame, text=gettextRcmdr("Show surface grid lines")), gridLinesCheckBox, sticky="w")
    tkgrid(tklabel(surfacesFrame, text=gettextRcmdr("Open new 3D graphics window")), rglOpenCheckBox, sticky="w")
    tkgrid(tklabel(surfacesFrame, text=gettextRcmdr("Surfaces to Fit"), fg="blue"), sticky="w")

    betaMultiplierLabel <- tklabel(surfacesFrame, text=gettextRcmdr("coef multiple = "))
    tkgrid(tklabel(surfacesFrame, text=gettextRcmdr("Linear least-squares")), linearLSCheckBox, betaMultiplierLabel, betaMultiplierField, sticky="w")
    tkgrid.configure(betaMultiplierLabel, sticky="e")

    tkgrid(tklabel(surfacesFrame, text=gettextRcmdr("Quadratic least-squares")), quadLSCheckBox, sticky="w")

    dfLabel <- tklabel(surfacesFrame, text=gettextRcmdr("df = "))
    tkgrid(tklabel(surfacesFrame, text=gettextRcmdr("Smooth regression")), nonparCheckBox, 
        dfLabel, dfNonparField, sticky="w")
    tkgrid.configure(dfLabel, sticky="e")

    tkgrid(tklabel(surfacesFrame, text=gettextRcmdr("Additive regression")), additiveCheckBox, 
        tklabel(surfacesFrame, text=gettextRcmdr("df(each term) = ")), dfAddField, sticky="w")
    tkgrid(surfacesFrame, sticky="w") 
    tkgrid(tklabel(bgFrame, text=gettextRcmdr("Background Color"), fg="blue"), sticky="w", columnspan=2)
    tkgrid(tklabel(bgFrame, text=gettextRcmdr("Black")), blackButton, sticky="w")
    tkgrid(tklabel(bgFrame, text=gettextRcmdr("White")), whiteButton, sticky="w")
    tkgrid(bgFrame, sticky="w")

    tkgrid(tklabel(residFrame, text=gettextRcmdr("Residuals Display"), fg="blue"), sticky="w", columnspan=2)
    tkgrid(tklabel(residFrame, text=gettextRcmdr("Line")), lineButton, sticky="w")
    tkgrid(tklabel(residFrame, text=gettextRcmdr("Square")), squareButton, sticky="w")
    tkgrid(residFrame, sticky="w")

    tkgrid(groupsFrame, sticky="w")
    tkgrid(buttonsFrame, stick="w")
    dialogSuffix(rows=5, columns=1)
    }

## The following function by Richard Heiberger, with small modifications by J. Fox
## with more modifications by Richard Heiberger.
## 2008-01-03 added conditions, layout, and multiple colors
Xyplot.HH <- function() {
    require("lattice")
    initializeDialog(title=gettextRcmdr("XY Conditioning Plot"))
    predictorFrame <- tkframe(top)
    predictorBox <- variableListBox(predictorFrame, Numeric(), title=gettextRcmdr("Explanatory variables (pick one or more)"), selectmode="multiple")
    responseBox <- variableListBox(predictorFrame, Numeric(), title=gettextRcmdr("Response variables (pick one or more)"), selectmode="multiple")
    cgFrame <- tkframe(top)
    conditionsBox <- variableListBox(cgFrame, Factors(), title=gettextRcmdr("Conditions '|' (pick zero or more)"), selectmode="multiple", initialSelection=FALSE)
    groupsBox <- variableListBox(cgFrame, Factors(), title=gettextRcmdr("Groups 'groups=' (pick zero or more)"), selectmode="multiple", initialSelection=FALSE)
    checkBoxes(frame="optionsFrame",
               boxes=c("auto.key", "outer"),
               initialValues=c(1,0),
               labels=gettextRcmdr(c("Automatically draw key", 
                "Different panels for different y~x combinations")))
    relationFrame <- tkframe(top)
    radioButtons(window=relationFrame,
                 name="x.relation",
                 buttons=c("same", "free", "sliced"),
                 labels=gettextRcmdr(c("Identical", "Free", "Same range")),
                 title=gettextRcmdr("X-Axis Scales in Different Panels"))
    radioButtons(window=relationFrame,
                 name="y.relation",
                 buttons=c("same", "free", "sliced"),
                 labels=gettextRcmdr(c("Identical", "Free", "Same range")),
                 title=gettextRcmdr("Y-Axis Scales in Different Panels"))

    scalarsFrame <- tkframe(top)

    layoutColumnsVar <- tclVar("")
    layoutColumnsEntry <- tkentry(scalarsFrame, width="6", textvariable=layoutColumnsVar)
    layoutRowsVar <- tclVar("")
    layoutRowsEntry <- tkentry(scalarsFrame, width="6", textvariable=layoutRowsVar)

    checkBoxes(frame="typeFrame",
               boxes=c("points", "lines"),
               initialValues=c(1,0),
               labels=gettextRcmdr(c("Points", 
                "Lines")))

    onOK <- function() {
        predictor <- getSelection(predictorBox)
        response <- getSelection(responseBox)
        conditions <- getSelection(conditionsBox)
        groups <- getSelection(groupsBox)
        closeDialog()
        if (0 == length(response)) {
            errorCondition(recall=Xyplot.HH, message=gettextRcmdr("At least one response variable must be selected."))
            return()
            }
        if (0 == length(predictor)) {
            errorCondition(recall=Xyplot.HH, message=gettextRcmdr("At least one explanatory variable must be selected."))
            return()
            }
        auto.key <- ("1" == tclvalue(auto.keyVariable))
        outer    <- ("1" == tclvalue(outerVariable))
        x.relation <- as.character(tclvalue(x.relationVariable))
        y.relation <- as.character(tclvalue(y.relationVariable))

        layoutColumns  <- as.numeric(tclvalue(layoutColumnsVar))
        layoutRows     <- as.numeric(tclvalue(layoutRowsVar))
        layout.command <- ""
        number.na <- is.na(layoutColumns) + is.na(layoutRows)

        if (number.na==1) {
            errorCondition(recall=Xyplot.HH,
                           message=gettextRcmdr("Both or neither layout values must be numbers."))
            return()
          }
        if (number.na==0) layout.command <- deparse(c(layoutColumns, layoutRows))
        
        .activeDataSet <- ActiveDataSet()

        
        
        conditions.command <-
          if (length(conditions)==0) {
            if (outer) {
              if (layout.command=="")
                paste(", layout=c(",
                      length(predictor),
                      ",",
                      length(response),
                      ")")
              else
                paste(", layout=", layout.command, sep="")
            }
            else
              if (layout.command != "")
                paste(", layout=", layout.command, sep="")
          }
          else {  ## (length(conditions)>0)
            if (outer) {
              condition.levels <- prod(sapply(conditions, d.f=get(.activeDataSet),
                                              function(g, d.f) length(levels(d.f[[g]]))))
              if (layout.command!="")
                paste(", layout=", layout.command, sep="")
              else
                paste(", layout=c(",
                      condition.levels,
                      "*",
                      length(predictor),
                      ",",
                      length(response),
                      ")",
                      ## ", between=list(x=c(0,0, 1, 0,0), y=1)",
                      ", between=list(x=c(",
                      paste(rep(c(rep(0, condition.levels-1), 1),
                                length=condition.levels*length(predictor)-1),
                            collapse=","),
                      "), y=1)")
            }
            else
              if (layout.command != "")
                paste(", layout=", layout.command, sep="")
          }
        
##         groups.command <-
##           if (length(groups)==1) paste(", groups=", groups, sep="")
##           else ""
        groups.command <- switch(as.character(length(groups)),
                                 "0"="",
                                 "1"=paste(", groups=", groups, sep=""),
                                 paste(", groups=interaction(",
                                       paste(groups, collapse=","),
                                       ")", sep=""))

        points.check <- ("1" == tclvalue(pointsVariable))
        lines.check  <- ("1" == tclvalue(linesVariable))
        if(!(points.check || lines.check)) {
            errorCondition(recall=Xyplot.HH,
                           message=gettextRcmdr("Choose at least one of points or lines."))
            return()
          }
        type.command <- paste(", type=",
                              deparse(c("p"[points.check], "l"[lines.check])),
                              sep="")
        
        xyplot.command <- paste("xyplot(",
                                paste(response, collapse=' + '),
                                " ~ ",
                                paste(predictor, collapse=' + '),
                                if (length(conditions) > 0)
                                   paste(" |",
                                         paste(conditions, collapse=' + ')
                                         ) else "",
                                if (outer) ", outer=TRUE",
                                conditions.command,
                                groups.command,
                                type.command,
                                ", pch=16",
                                if (auto.key) ", auto.key=list(border=TRUE), par.settings=simpleTheme(pch=16)" else "",
                                paste(", scales=list(x=list(relation='",
                                      x.relation,
                                      "'), y=list(relation='",
                                      y.relation,
                                      "'))", sep=""),
                                ", data=", .activeDataSet, ')', sep="")
        doItAndPrint(xyplot.command)
        activateMenus()
        tkfocus(CommanderWindow())
        }
    OKCancelHelp(helpSubject="xyplot")
    tkgrid(getFrame(predictorBox), getFrame(responseBox),
           columnspan=1, sticky="w")
    tkgrid(predictorFrame, sticky="w")
    tkgrid(getFrame(conditionsBox),
           tklabel(cgFrame, text=gettextRcmdr("           ")),
           getFrame(groupsBox),
           columnspan=1, sticky="w")
    tkgrid(cgFrame, sticky="w")

    tkgrid(tklabel(top, text=gettextRcmdr("Options"), fg="blue"), sticky="w")
    tkgrid(optionsFrame, sticky="w")
    tkgrid(tklabel(top, text=gettextRcmdr("Plot Type (one or both)"), fg="blue"), sticky="w")
    tkgrid(typeFrame, sticky="w")

    tkgrid(x.relationFrame, y.relationFrame, columnspan=2, sticky="w")
    tkgrid(relationFrame, sticky="w")

    tkgrid(tklabel(top, text=gettextRcmdr("Layout"), fg="blue"),
           sticky="w")
    tkgrid(tklabel(scalarsFrame, text=gettextRcmdr("number of columns:")), layoutColumnsEntry, sticky="w")
    tkgrid(tklabel(scalarsFrame, text=gettextRcmdr("number of rows:")), layoutRowsEntry, sticky="w")
    tkgrid(scalarsFrame, sticky="w")

    tkgrid(buttonsFrame, columnspan=2, sticky="w")
    dialogSuffix(rows=6, columns=2)
    }
## source("~/HH-R.package/RcmdrPlugin.HH/R/Xyplot.HH.R")
"analyzeTwoWayTable.HH" <-
function(){
    initializeDialog(title=gettextRcmdr("Analyze Two-Way Table"))
    variablesFrame <- tkframe(top)
    onOK <- function() {
        rowPct <- tclvalue(rowPercentsVariable)
        colPct <- tclvalue(colPercentsVariable)
        totPct <- tclvalue(totPercentsVariable)

        chisq <- tclvalue(chisqTestVariable)
        chisqComp <- tclvalue(chisqCompVariable)
        chiComp <- tclvalue(chiCompVariable)
        expected <- tclvalue(expFreqVariable)
        fisher <- tclvalue(fisherTestVariable)
        closeDialog()

        ## .Table <- data.matrix(ActiveDataSet())
        command <- ActiveDataSet()
        logger(paste(".Table <- ", command, sep=""))
        assign(".Table", justDoIt(command), envir=.GlobalEnv)
        doItAndPrint(".Table")

        if (rowPct == 1) doItAndPrint("rowPercents(.Table) # Row Percentages")
        if (colPct == 1) doItAndPrint("colPercents(.Table) # Column Percentages")        
        if (totPct == 1) doItAndPrint("totPercents2.HH(.Table) # Total Percentages")        

        if (chisq == 1) {
            command <- "chisq.test(.Table, correct=FALSE)"
            logger(paste(".Test <- ", command, sep=""))
            assign(".Test", justDoIt(command), envir=.GlobalEnv)
            doItAndPrint(".Test")
            if (chisqComp == 1) doItAndPrint("round(.Test$residuals^2, 2) # Chi-square Components")
            if (chiComp == 1) doItAndPrint("round(.Test$residuals, 2) # Chi Components (residuals)")
            if (expected == 1) doItAndPrint("round(.Test$expected, 2) # Expected Counts")
            warnText <- NULL
            if (0 < (nlt1 <- sum(.Test$expected < 1))) warnText <- paste(nlt1,
                gettextRcmdr("expected frequencies are less than 1"))
            if (0 < (nlt5 <- sum(.Test$expected < 5))) warnText <- paste(warnText, "\n", nlt5,
                gettextRcmdr(" expected frequencies are less than 5"), sep="")
            if (!is.null(warnText)) Message(message=warnText,
                type="warning")
            logger("remove(.Test)") 
            remove(.Test, envir=.GlobalEnv) 
            }
        if (fisher == 1) doItAndPrint("fisher.test(.Table)")
        logger("remove(.Table)") 
        remove(.Table, envir=.GlobalEnv)                                                      
        tkfocus(CommanderWindow())
        }
    OKCancelHelp(helpSubject="chisq.test")

    checkBoxes(frame="percentsFrame",
               boxes=c("rowPercents",
                 "colPercents",
                 "totPercents"), 
               initialValues=c("0","0","0"), 
               labels=gettextRcmdr(c(
                 "Row percentages",
                 "Column percentages",
                 "Total percentages")))

    checkBoxes(frame="testsFrame",
               boxes=c(
                 "chisqTest",
                 "chisqComp",
                 "chiComp",
                 "expFreq",
                 "fisherTest"),
               initialValues=c("1", "1", "0", "0", "0"),
               labels=gettextRcmdr(c(
                 "Chi-square test of independence",
                 "Print chi-square components",
                 "Print chi components (residuals)",
                 "Print expected frequencies",
                 "Fisher's exact test")))

    tkgrid(variablesFrame, sticky="w")
    tkgrid(tklabel(top, text=gettextRcmdr("Compute Percentages"), fg="blue"), sticky="w")
    tkgrid(percentsFrame, sticky="w")
    tkgrid(tklabel(top, text=gettextRcmdr("Hypothesis Tests"), fg="blue"), sticky="w")
    tkgrid(testsFrame, sticky="w")
    tkgrid(buttonsFrame, sticky="w")
    dialogSuffix(rows=4, columns=1)
  }

"anovaTableI.HH" <-
function(){
    .activeModel <- ActiveModel()
    if (!checkMethod("anova", .activeModel)) {
        errorCondition(message=gettextRcmdr("There is no appropriate anova method for a model of this class."))
        return()
        }
    doItAndPrint(paste("anova(", .activeModel, ")", sep=""))
    }

## this is an exact copy of John Fox's anovaTable from Rcmdr/R/model-menu.R

anovaTableII.HH <- function(){
    require("car")
    .activeModel <- ActiveModel()
    if (is.null(.activeModel)) return()
    if (!checkMethod("Anova", .activeModel)) {
        errorCondition(message=gettextRcmdr("There is no appropriate Anova method for a model of this class."))
        return()
        }
    doItAndPrint(paste("Anova(", .activeModel, ")", sep=""))
    }
"bestSubsetsRegressionModel.HH" <-
function(){
    require("leaps")
    initializeDialog(title=gettextRcmdr("Best Subsets Regression"))
    variablesFrame <- tkframe(top)
    .numeric <- Numeric()
    xBox <- variableListBox(variablesFrame, .numeric, selectmode="multiple", 
        title=gettextRcmdr("Explanatory variables (pick one or more)"), listHeight=7)
    yBox <- variableListBox(variablesFrame, .numeric, title=gettextRcmdr("Response variable (pick one)"), listHeight=7)
    UpdateModelNumber()
    modelName <- tclVar(paste("RegModel.", getRcmdr("modelNumber"), sep=""))
    subsetsName <- tclVar(paste("Subsets.", getRcmdr("modelNumber"), sep=""))
    modelFrame <- tkframe(top)
    model <- tkentry(modelFrame, width="20", textvariable=subsetsName)
    subsetBox()
    nbestName <- tclVar("2")
    nbestFrame <- tkframe(top)
    nbest <- tkentry(nbestFrame, width="3", textvariable=nbestName)

    radioButtons(name="statistic",
                 buttons=c("rsq", "rss", "adjr2", "cp", "bic", "stderr"), 
                 values=c("rsq", "rss", "adjr2", "cp", "bic", "stderr"),
                 initialValue="adjr2", 
                 labels=gettextRcmdr(c("R Square", "Residual Sum of Squares",
                   "Adjusted R^2", "Cp", "BIC", "Standard Error")),
                 title=gettextRcmdr("Statistic to plot"))

    onOK <- function() {
        x <- getSelection(xBox)
        y <- getSelection(yBox)
        closeDialog()
        if (0 == length(y)) {
            UpdateModelNumber(-1)
            errorCondition(recall=bestSubsetsRegressionModel.HH, message=gettextRcmdr("You must select a response variable."))
            return()
            }
        if (0 == length(x)) {
            UpdateModelNumber(-1)
            errorCondition(recall=bestSubsetsRegressionModel.HH, message=gettextRcmdr("No explanatory variables selected."))
            return()
            }        
        if (is.element(y, x)) {
            UpdateModelNumber(-1)
            errorCondition(recall=bestSubsetsRegressionModel.HH, message=gettextRcmdr("Response and explanatory variables must be different."))
            return()
            }
        subset <- tclvalue(subsetVariable)
        if (trim.blanks(subset) == gettextRcmdr("<all valid cases>") || trim.blanks(subset) == ""){
            subset <- ""
            putRcmdr("modelWithSubset", FALSE)
            }
        else{
            subset <- paste(", subset=", subset, sep="")
            putRcmdr("modelWithSubset", TRUE)
            }
        subsetsValue <- trim.blanks(tclvalue(subsetsName))
        if (!is.valid.name(subsetsValue)){
            UpdateModelNumber(-1)
            errorCondition(recall=bestSubsetsRegressionModel.HH, message=sprintf(gettextRcmdr('"%s" is not a valid name.'), subsetsValue))
            return()
            }
        if (is.element(subsetsValue, listLinearModels())) {
            if ("no" == tclvalue(checkReplace(subsetsValue, type=gettextRcmdr("Model")))) {
                UpdateModelNumber(-1)
                bestSubsetsRegressionModel.HH()
                return()
                }
            }

        nbestValue <- as.integer(tclvalue(nbestName))
        if (!((is.integer(nbestValue)) && nbestValue>0 && nbestValue<=length(x))) {
            UpdateModelNumber(-1)
            errorCondition(recall=bestSubsetsRegressionModel.HH, message=sprintf(gettextRcmdr('"%s" is not a valid nbest value.'), nbestValue))
            return()
            }
        statisticValue <- tclvalue(statisticVariable)

        command <- paste("regsubsets(", y, "~", paste(x, collapse="+"),
                         ", data=", ActiveDataSet(), subset,
                         ", nbest=", nbestValue,
                         ")", sep="")
        logger(paste(subsetsValue, " <- ", command, sep=""))
        assign(subsetsValue, justDoIt(command), envir=.GlobalEnv)

        command <- paste("summary_HH(", subsetsValue, ")", sep="")
        summaryValue <- paste(subsetsValue, "Summary", sep=".")
        logger(paste(summaryValue, " <- ", command, sep=""))
        assign(summaryValue, justDoIt(command), envir=.GlobalEnv)
        ## summaries <-
        doItAndPrint(summaryValue)

        command <- paste("plot(", summaryValue, ", statistic='", statisticValue, "', legend=FALSE)", sep="")
        logger(command)
        justDoIt(command)
        
        bringToTop()
        .nmax <- attr(get(summaryValue, envir=.GlobalEnv), "n.max.adjr2")
        ## .nmax <- attr(summaries, "n.max.adjr2")
        modelValue <- paste(trim.blanks(tclvalue(modelName)), .nmax, sep=".")

        command <- paste("lm.regsubsets(", subsetsValue, ", ", .nmax, ")", sep="")
        logger(paste(modelValue, " <- ", command, "  ## subset ", .nmax, " has largest adjr2", sep=""))
        assign(modelValue , justDoIt(command), envir=.GlobalEnv)
        doItAndPrint(paste("summary(", modelValue, ")", sep=""))

        activeModel(modelValue)
        tkfocus(CommanderWindow())
        }
    OKCancelHelp(helpSubject="regsubsets", model=TRUE)
    tkgrid(tklabel(modelFrame, text=gettextRcmdr("Enter name for model:")), model, sticky="w")
    tkgrid(modelFrame, sticky="w")
    tkgrid(getFrame(yBox), tklabel(variablesFrame, text="    "), getFrame(xBox), sticky="nw")
    tkgrid(variablesFrame, sticky="w")    

    tkgrid(tklabel(nbestFrame, text=gettextRcmdr("Number of subsets of each size to record:")), nbest, sticky="w")
    tkgrid(nbestFrame, sticky="w")

    tkgrid(statisticFrame, sticky="w")

    tkgrid(subsetFrame, sticky="w")
    tkgrid(buttonsFrame, stick="w")
    tkgrid.configure(helpButton, sticky="e")
    dialogSuffix(rows=5, columns=1)
    }

CloseCommanderRestart <- function() {
  closeCommander(ask.save=getRcmdr("ask.on.exit"))
  Commander()
}

CloseCommanderNoQuestionRestart <- function() {
  closeCommander(ask=FALSE)
  Commander()
}
"confidenceIntervalsPlot" <-
function(){
  require("lattice")
  initializeDialog(title=gettextRcmdr("Confidence Intervals in Simple Linear Regression"))
    variablesFrame <- tkframe(top)
    .numeric <- Numeric()
    xBox <- variableListBox(variablesFrame, .numeric,
        title=gettextRcmdr("Explanatory variables (pick one)"))
    yBox <- variableListBox(variablesFrame, .numeric, title=gettextRcmdr("Response variable (pick one)"))
    UpdateModelNumber()
    modelName <- tclVar(paste("RegModel.", getRcmdr("modelNumber"), sep=""))
    modelFrame <- tkframe(top)
    model <- tkentry(modelFrame, width="20", textvariable=modelName)
    subsetBox()
    onOK <- function(){
        x <- getSelection(xBox)
        y <- getSelection(yBox)
        closeDialog()
        if (0 == length(y)) {
            UpdateModelNumber(-1)
            errorCondition(recall=confidenceIntervalsPlot, message=gettextRcmdr("You must select a response variable."))
            return()
            }
        if (0 == length(x)) {
            UpdateModelNumber(-1)
            errorCondition(recall=confidenceIntervalsPlot, message=gettextRcmdr("No explanatory variables selected."))
            return()
            }        
        if (is.element(y, x)) {
            UpdateModelNumber(-1)
            errorCondition(recall=confidenceIntervalsPlot, message=gettextRcmdr("Response and explanatory variables must be different."))
            return()
            }
        subset <- tclvalue(subsetVariable)
        if (trim.blanks(subset) == gettextRcmdr("<all valid cases>") || trim.blanks(subset) == ""){
            subset <- ""
            putRcmdr("modelWithSubset", FALSE)
            }
        else{
            subset <- paste(", subset=", subset, sep="")
            putRcmdr("modelWithSubset", TRUE)
            }
        modelValue <- trim.blanks(tclvalue(modelName))
        if (!is.valid.name(modelValue)){
            UpdateModelNumber(-1)
            errorCondition(recall=confidenceIntervalsPlot, message=sprintf(gettextRcmdr('"%s" is not a valid name.'), modelValue))
            return()
            }
        if (is.element(modelValue, listLinearModels())) {
            if ("no" == tclvalue(checkReplace(modelValue, type=gettextRcmdr("Model")))){
                UpdateModelNumber(-1)
                confidenceIntervalsPlot()
                return()
                }
            }
        command <- paste("lm(", y, "~", paste(x, collapse="+"),
            ", data=", ActiveDataSet(), subset, ")", sep="")
        logger(paste(modelValue, " <- ", command, sep=""))
        assign(modelValue, justDoIt(command), envir=.GlobalEnv)
        doItAndPrint(paste("summary(", modelValue, ")", sep=""))
        activeModel(modelValue)
        doItAndPrint(paste("ci.plot(", modelValue, ")", sep=""))
        tkfocus(CommanderWindow())
        }
    OKCancelHelp(helpSubject="ci.plot", model=TRUE)
    tkgrid(tklabel(modelFrame, text=gettextRcmdr("Enter name for model:")), model, sticky="w")
    tkgrid(modelFrame, sticky="w")
    tkgrid(getFrame(yBox), tklabel(variablesFrame, text="    "), getFrame(xBox), sticky="nw")
    tkgrid(variablesFrame, sticky="w")    
    tkgrid(subsetFrame, sticky="w")
    tkgrid(buttonsFrame, stick="w")
    tkgrid.configure(helpButton, sticky="e")
    dialogSuffix(rows=4, columns=1)
    }

"enterTable.HH" <-
function(){
    require("abind")
    env <- environment()
    initializeDialog(title=gettextRcmdr("Enter Two-Way Table"))
    outerTableFrame <- tkframe(top)
    assign(".tableFrame", tkframe(outerTableFrame), envir=env)
    setUpTable <- function(...){
        tkdestroy(get(".tableFrame", envir=env))
        assign(".tableFrame", tkframe(outerTableFrame), envir=env)
        nrows <- as.numeric(tclvalue(rowsValue))
        ncols <- as.numeric(tclvalue(colsValue))
        make.col.names <- "tklabel(.tableFrame, text='')"
        for (j in 1:ncols) {
            col.varname <- paste(".colname.", j, sep="")
            assign(col.varname, tclVar(j), envir=env)
            make.col.names <- paste(make.col.names, ", ", "tkentry(.tableFrame, width='5', textvariable=", 
                    col.varname, ")", sep="")
            }
        eval(parse(text=paste("tkgrid(", make.col.names, ")", sep="")), envir=env)
        for (i in 1:nrows){   
            varname <- paste(".tab.", i, ".1", sep="") 
            assign(varname, tclVar("") , envir=env)
            row.varname <- paste(".rowname.", i, sep="")
            assign(row.varname, tclVar(i), envir=env)
            make.row <- paste("tkentry(.tableFrame, width='5', textvariable=",
                row.varname, ")", sep="")
            make.row <- paste(make.row, ", ", "tkentry(.tableFrame, width='5', textvariable=", 
                varname, ")", sep="")
            for (j in 2:ncols){
                varname <- paste(".tab.", i, ".", j, sep="")
                assign(varname, tclVar(""), envir=env)
                make.row <- paste(make.row, ", ", "tkentry(.tableFrame, width='5', textvariable=", 
                    varname, ")", sep="")
                }
            eval(parse(text=paste("tkgrid(", make.row, ")", sep="")), envir=env)
            }
        tkgrid(get(".tableFrame", envir=env), sticky="w")
        }
    rowColFrame <- tkframe(top)
    rowsValue <- tclVar("2")
    rowsSlider <- tkscale(rowColFrame, from=2, to=10, showvalue=FALSE, variable=rowsValue,
        resolution=1, orient="horizontal", command=setUpTable)
    rowsShow <- tklabel(rowColFrame, textvariable=rowsValue, width=2, justify="right")
    colsValue <- tclVar("2")
    colsSlider <- tkscale(rowColFrame, from=2, to=10, showvalue=FALSE, variable=colsValue,
        resolution=1, orient="horizontal", command=setUpTable)
    colsShow <- tklabel(rowColFrame, textvariable=colsValue, width=2, justify="right")
    onOK <- function() {
        rowPct <- tclvalue(rowPercentsVariable)
        colPct <- tclvalue(colPercentsVariable)
        totPct <- tclvalue(totPercentsVariable)

        nrows <- as.numeric(tclvalue(rowsValue))
        ncols <- as.numeric(tclvalue(colsValue))
        cell <- 0
        counts <- rep(NA, nrows*ncols)
        row.names <- rep("", nrows)
        col.names <- rep("", ncols)
        for (i in 1:nrows) row.names[i] <- 
            eval(parse(text=paste("tclvalue(", paste(".rowname.", i, sep=""),")", sep="")))
        for (j in 1:ncols) col.names[j] <- 
            eval(parse(text=paste("tclvalue(", paste(".colname.", j, sep=""),")", sep="")))
        for (i in 1:nrows){
            for (j in 1:ncols){
                cell <- cell+1
                varname <- paste(".tab.", i, ".", j, sep="")
                counts[cell] <- as.numeric(eval(parse(text=paste("tclvalue(", varname,")", sep=""))))
                }
            }
        counts <- na.omit(counts)
        if (length(counts) != nrows*ncols){
            errorCondition(recall=enterTable.HH, message=sprintf(gettextRcmdr("Number of valid entries (%d)\nnot equal to number of rows (%d) * number of columns (%d)."), length(counts), nrows, ncols))
            return()
            }
        if (length(unique(row.names)) != nrows){
            errorCondition(recall=enterTable.HH, message=gettextRcmdr("Row names are not unique."))
            return()
            }     
        if (length(unique(col.names)) != ncols){
            errorCondition(recall=enterTable.HH, message=gettextRcmdr("Column names are not unique."))
            return()
            }     
##        percents <- as.character(tclvalue(percentsVariable))
        chisq <- tclvalue(chisqTestVariable)
        chisqComp <- tclvalue(chisqCompVariable)
        chiComp <- tclvalue(chiCompVariable)
        expected <- tclvalue(expFreqVariable)
        fisher <- tclvalue(fisherTestVariable)
        closeDialog()

        command <- paste("matrix(c(", paste(counts, collapse=","), "), ", nrows, ", ", ncols,
            ", byrow=TRUE)", sep="")
        assign(".Table", justDoIt(command), envir=.GlobalEnv)
        logger(paste(".Table <- ", command, sep=""))
        command <- paste("c(",paste(paste("'", row.names, "'", sep=""), collapse=", "), ")", sep="")
        justDoIt(paste("rownames(.Table) <- ", command, sep=""))
        logger(paste("rownames(.Table) <- ", command, sep=""))
        command <- paste("c(",paste(paste("'", col.names, "'", sep=""), collapse=", "), ")", sep="")
        justDoIt(paste("colnames(.Table) <- ", command, sep=""))
        logger(paste("colnames(.Table) <- ", command, sep=""))
        doItAndPrint(".Table  # Counts")

        if (rowPct == 1) doItAndPrint("rowPercents(.Table) # Row Percentages")
        if (colPct == 1) doItAndPrint("colPercents(.Table) # Column Percentages")        
        if (totPct == 1) doItAndPrint("totPercents2.HH(.Table) # Total Percentages")        

 
        if (chisq == 1) {
            command <- "chisq.test(.Table, correct=FALSE)"
            logger(paste(".Test <- ", command, sep=""))
            assign(".Test", justDoIt(command), envir=.GlobalEnv)
            doItAndPrint(".Test")
            if (chisqComp == 1) doItAndPrint("round(.Test$residuals^2, 2) # Chi-square Components")
            if (chiComp == 1) doItAndPrint("round(.Test$residuals, 2) # Chi Components (residuals)")
            if (expected == 1) doItAndPrint("round(.Test$expected, 2) # Expected Counts")
##            if (expected == 1) doItAndPrint(".Test$expected # Expected Counts")
            warnText <- NULL
            if (0 < (nlt1 <- sum(.Test$expected < 1))) warnText <- paste(nlt1,
                gettextRcmdr("expected frequencies are less than 1"))
            if (0 < (nlt5 <- sum(.Test$expected < 5))) warnText <- paste(warnText, "\n", nlt5,
                gettextRcmdr(" expected frequencies are less than 5"), sep="")
            if (!is.null(warnText)) Message(message=warnText,
                type="warning")
            logger("remove(.Test)") 
            remove(.Test, envir=.GlobalEnv) 
            }
        if (fisher == 1) doItAndPrint("fisher.test(.Table)")
        logger("remove(.Table)") 
        remove(.Table, envir=.GlobalEnv)                                                      
        tkfocus(CommanderWindow())
        }
    OKCancelHelp(helpSubject="chisq.test")

    checkBoxes(frame="percentsFrame",
               boxes=c("rowPercents",
                 "colPercents",
                 "totPercents"), 
               initialValues=c("0","0","0"), 
               labels=gettextRcmdr(c(
                 "Row percentages",
                 "Column percentages",
                 "Total percentages")))
    
##     radioButtons(name="percents",
##                  buttons=c("rowPercents",
##                    "columnPercents",
##                    "nonePercents"),
##                  values=c("row", "column", "none"),
##                  initialValue="none",
##                  labels=gettextRcmdr(c
##                    ("Row percentages",
##                     "Column percentages",
##                     "No percentages")),
##                  title=gettextRcmdr("Compute Percentages"))
    
    checkBoxes(frame="testsFrame",
               boxes=c(
                 "chisqTest",
                 "chisqComp",
                 "chiComp",
                 "expFreq",
                 "fisherTest"),
               initialValues=c("1", "1", "0", "0", "0"),
               labels=gettextRcmdr(c(
                 "Chi-square test of independence",
                 "Print chi-square components",
                 "Print chi components (residuals)",
                 "Print expected frequencies",
                 "Fisher's exact test")))

##     checkBoxes(frame="testsFrame",
##                boxes=c("chisq", "expFreq", "fisher"),
##                initialValues=c("1", "0", "0"),                               
##                labels=gettextRcmdr(c("Chi-square test of independence",
##                  "Print expected frequencies",
##                  "Fisher's exact test")))
    
    tkgrid(tklabel(rowColFrame,
                   text=gettextRcmdr("Number of Rows:")),
           rowsSlider, rowsShow, sticky="w")
    tkgrid(tklabel(rowColFrame,
                   text=gettextRcmdr("Number of Columns:")),
           colsSlider, colsShow, sticky="w")
    tkgrid(rowColFrame, sticky="w")
    tkgrid(tklabel(top,
                   text=gettextRcmdr("Enter counts:"),
                   fg="blue"), sticky="w")
    tkgrid(outerTableFrame, sticky="w")
    tkgrid(tklabel(top, text=gettextRcmdr("Compute Percentages"), fg="blue"), sticky="w")
    tkgrid(percentsFrame, sticky="w")
    tkgrid(tklabel(top,
                   text=gettextRcmdr("Hypothesis Tests"),
                   fg="blue"), sticky="w")
    tkgrid(testsFrame, sticky="w")
    tkgrid(buttonsFrame, columnspan=2, sticky="w")
    dialogSuffix(rows=7, columns=2)
    }

normal.and.t.hypotheses.plot <- function() {
  initializeDialog(title=gettextRcmdr("Normal and t Distributions"))
  muVar <- tclVar("")
  muEntry <- tkentry(top, width="6", textvariable=muVar)
  sigmaVar <- tclVar("")
  sigmaEntry <- tkentry(top, width="6", textvariable=sigmaVar)
  degfreeVar <- tclVar("")
  degfreeEntry <- tkentry(top, width="6", textvariable=degfreeVar)
  nVar <- tclVar("")
  nEntry <- tkentry(top, width="6", textvariable=nVar)
  leftAlphaVar <- tclVar("")
  leftAlphaEntry <- tkentry(top, width="6", textvariable=leftAlphaVar)
  rightAlphaVar <- tclVar("")
  rightAlphaEntry <- tkentry(top, width="6", textvariable=rightAlphaVar)
  obsValueVar <- tclVar("")
  obsValueEntry <- tkentry(top, width="6", textvariable=obsValueVar)
  muAltVar <- tclVar("")
  muAltEntry <- tkentry(top, width="6", textvariable=muAltVar)
  ymaxVar <- tclVar("")
  ymaxEntry <- tkentry(top, width="6", textvariable=ymaxVar)

  radioButtons(name="hypoth",
               buttons=c("Hypoth", "Conf"),
               labels=gettextRcmdr(c("Hypothesis Test", "Confidence Interval")),
               title=gettextRcmdr("Hypothesis or Confidence"))

  onOK <- function() {
    closeDialog()
    mu <- as.numeric(tclvalue(muVar))
    if (is.na(mu)) mu <- 0
    sigma <- as.numeric(tclvalue(sigmaVar))
    if (!is.na(sigma) && sigma <= 0) {
      errorCondition(recall=normal.and.t.hypotheses.plot,
                     message=gettextRcmdr("Standard deviation must be positive."))
            return()
            }
    degfree <- as.numeric(tclvalue(degfreeVar))
    if (!is.na(degfree) && degfree <= 0) {
      errorCondition(recall=normal.and.t.hypotheses.plot,
                     message=gettextRcmdr("Degrees of freedom must be positive."))
            return()
            }
    n <- as.numeric(tclvalue(nVar))
    if (!is.na(n) && n <= 0) {
      errorCondition(recall=normal.and.t.hypotheses.plot,
                     message=gettextRcmdr("Sample size must be positive."))
            return()
            }
    leftAlpha <- as.numeric(tclvalue(leftAlphaVar))
    if (!is.na(leftAlpha) && (leftAlpha < 0 || 1 < leftAlpha)) {
      errorCondition(recall=normal.and.t.hypotheses.plot,
                     message=gettextRcmdr("leftAlpha must be between 0 and 1."))
            return()
            }
    rightAlpha <- as.numeric(tclvalue(rightAlphaVar))
    if (!is.na(rightAlpha) && (rightAlpha < 0 || 1 < rightAlpha)) {
      errorCondition(recall=normal.and.t.hypotheses.plot,
                     message=gettextRcmdr("rightAlpha must be between 0 and 1."))
            return()
            }
    if (!(is.na(leftAlpha) || is.na(rightAlpha))&& leftAlpha + rightAlpha > 1) {
      errorCondition(recall=normal.and.t.hypotheses.plot,
                     message=gettextRcmdr("left Alpha + rightAlpha must be less than 1."))
            return()
            }

    hypoth <- as.character(tclvalue(hypothVariable))
    obsValue <- as.numeric(tclvalue(obsValueVar))
    if (hypoth=="Conf" && is.na(obsValue)) {
      errorCondition(recall=normal.and.t.hypotheses.plot,
                     message=gettextRcmdr("Confidence Intervals require an observed mean."))
            return()
            }
    
    muAlt <- as.numeric(tclvalue(muAltVar))
    ymax <- as.numeric(tclvalue(ymaxVar))
    if (!is.na(ymax) && ymax <= 0) {
      errorCondition(recall=normal.and.t.hypotheses.plot,
                     message=gettextRcmdr("ymax must be positive."))
            return()
            }
    
    command <- paste('normal.and.t.dist(',
                     'std.dev=', sigma,
                     ', n=', n,
                     ', deg.freedom=', degfree,
                     ', mu.H0=', mu,
                     ', xmin=', NA,
                     ', xmax=', NA,
                     ', fx.min=', NA,
                     ', fx.max=', ymax,
                     ', Use.alpha.right=', !is.na(rightAlpha),
                     ', alpha.right=', rightAlpha,
                     ', Use.alpha.left=', !is.na(leftAlpha),
                     ', alpha.left=', leftAlpha,
                     ', Use.mu.H1=', !is.na(muAlt),
                     ', mu.H1=', muAlt,
                     ', Use.obs.mean=', !is.na(obsValue),
                     ', obs.mean=', obsValue,
                     ', hypoth.or.conf="', hypoth, '"',
                     ', col.mean=', '"lime green"',
                     ', col.alpha=', '"blue"',
                     ', col.beta=', '"red"',
                     ', col.conf=', '"pale green"',
                     ', col.conf.arrow=', '"dark green"',
                     ', col.mean.label=', '"lime green"',
                     ', col.alpha.label=', '"blue"',
                     ', col.beta.label=', '"red"',
                     ', col.conf.label=', '"dark green"',
                     ', cex.crit=', 1.2,
                     ')', sep='') 
        doItAndPrint(command)
      
      tkfocus(CommanderWindow())
    }
  OKCancelHelp(helpSubject=if (is.na(as.numeric(tclvalue(degfreeVar)))) "dnorm" else "dt")
  tkgrid(tklabel(top, text=gettextRcmdr("mu (mean)")), muEntry, sticky="e")
  tkgrid(tklabel(top, text=gettextRcmdr("standard deviation (sigma or s)")), sigmaEntry, sticky="e")
  tkgrid(tklabel(top, text=gettextRcmdr("df (degrees of freedom)")), degfreeEntry, sticky="e")
  tkgrid(tklabel(top, text=gettextRcmdr("n (sample size)")), nEntry, sticky="e")
  tkgrid(tklabel(top, text=gettextRcmdr("left alpha")), leftAlphaEntry, sticky="e")
  tkgrid(tklabel(top, text=gettextRcmdr("right alpha")), rightAlphaEntry, sticky="e")
  tkgrid(tklabel(top, text=gettextRcmdr("Observed Value")), obsValueEntry, sticky="e")
  tkgrid(tklabel(top, text=gettextRcmdr("mu (Alternate Hypothesis)")), muAltEntry, sticky="e")
  tkgrid(tklabel(top, text=gettextRcmdr("ymax (right-hand side)")), ymaxEntry, sticky="e")
  tkgrid.configure(muEntry, sticky="w")
  tkgrid.configure(sigmaEntry, sticky="w")
  tkgrid.configure(degfreeEntry, sticky="w")
  tkgrid.configure(nEntry, sticky="w")
  tkgrid.configure(leftAlphaEntry, sticky="w")
  tkgrid.configure(rightAlphaEntry, sticky="w")
  tkgrid.configure(obsValueEntry, sticky="w")
  tkgrid.configure(muAltEntry, sticky="w")
  tkgrid.configure(ymaxEntry, sticky="w")
  tkgrid(hypothFrame, sticky="w")
  tkgrid(buttonsFrame, columnspan=2, sticky="w")
  dialogSuffix(rows=10, columns=2, focus=muEntry)
}

## source("~/HH-R.package/RcmdrPlugin.HH/R/normal.and.t.hypotheses.plot.R")
"scatter3d.HH" <-
function (x, y, z,
            xlab = deparse(substitute(x)),
            ylab = deparse(substitute(y)), 
            zlab = deparse(substitute(z)),
            revolutions = 0,
            bg.col = c("white", "black"),
            axis.col = if (bg.col == "white") "black" else "white", 
            surface.col = c("blue", "green", "orange", "magenta", "cyan", 
              "red", "yellow", "gray"),
            neg.res.col = "red", pos.res.col = "green", 
            point.col = "yellow", text.col = axis.col,
            grid.col = if (bg.col == "white") "black" else "gray",
            fogtype = c("exp2", "linear", "exp", "none"),
            residuals = (length(fit) == 1), surface = TRUE, 
            grid = TRUE, grid.lines = 26, df.smooth = NULL, df.additive = NULL, 
            sphere.size = 1, threshold = 0.01,
            speed = 1, fov = 60, fit = "linear", 
            groups = NULL, parallel = TRUE, model.summary = FALSE,
            squares=FALSE, square.color="gray", coef.ratio=1)
{
    require(rgl)
    require(mgcv)
    summaries <- list()
    if ((!is.null(groups)) && (nlevels(groups) > length(surface.col))) 
        stop(sprintf(gettextRcmdr("Number of groups (%d) exceeds number of colors (%d)."), 
            nlevels(groups), length(surface.col)))
    if ((!is.null(groups)) && (!is.factor(groups))) 
        stop(gettextRcmdr("groups variable must be a factor."))
    bg.col <- match.arg(bg.col)
    fogtype <- match.arg(fogtype)
    if ((length(fit) > 1) && residuals && surface) 
        stop(gettextRcmdr("cannot plot both multiple surfaces and residuals"))
    xlab
    ylab
    zlab
    rgl.clear()
    rgl.viewpoint(fov = fov)
    rgl.bg(col = bg.col, fogtype = fogtype)
    valid <- if (is.null(groups)) 
        complete.cases(x, y, z)
    else complete.cases(x, y, z, groups)
    x <- x[valid]
    y <- y[valid]
    z <- z[valid]
    if (!is.null(groups)) 
        groups <- groups[valid]
    x <- (x - min(x))/(max(x) - min(x))
    y <- (y - min(y))/(max(y) - min(y))
    z <- (z - min(z))/(max(z) - min(z))
    size <- sphere.size * ((100/length(x))^(1/3)) * 0.015
    if (is.null(groups)) {
        if (size > threshold) 
            rgl.spheres(x, y, z, color = point.col, radius = size)
        else rgl.points(x, y, z, color = point.col)
    }
    else {
        if (size > threshold) 
            rgl.spheres(x, y, z, color = surface.col[as.numeric(groups)], 
                radius = size)
        else rgl.points(x, y, z, color = surface.col[as.numeric(groups)])
    }
    rgl.lines(c(0, 1), c(0, 0), c(0, 0), color = axis.col)
    rgl.lines(c(0, 0), c(0, 1), c(0, 0), color = axis.col)
    rgl.lines(c(0, 0), c(0, 0), c(0, 1), color = axis.col)
    rgl.texts(1, 0, 0, xlab, adj = 1, color = text.col)
    rgl.texts(0, 1, 0, ylab, adj = 1, color = text.col)
    rgl.texts(0, 0, 1, zlab, adj = 1, color = text.col)
    if (surface) {
        vals <- seq(0, 1, length = grid.lines)
        dat <- expand.grid(x = vals, z = vals)
        for (i in 1:length(fit)) {
            f <- match.arg(fit[i], c("linear", "quadratic", "smooth", 
                "additive"))
            if (is.null(groups)) {
                mod <-
                  switch(f,
                         linear = lm(y ~ x + z),
                         quadratic = lm(y ~ (x + z)^2 + I(x^2) + I(z^2)),
                         smooth =
                         if (is.null(df.smooth)) gam(y ~ s(x, z))
                         else gam(y ~ s(x, z, fx = TRUE, k = df.smooth)), 
                         additive =
                         if (is.null(df.additive)) gam(y ~ s(x) + s(z))
                         else gam(y ~ s(x, fx=TRUE, k=df.additive[1]+1)
                                  + s(z, fx=TRUE, k=(rev(df.additive+1)[1]+1))))
                if (model.summary) 
                  summaries[[f]] <- summary(mod)
                if (coef.ratio != 1) ## change the coefficients
                  mod$coefficients <- coef.ratio * mod$coefficients
                yhat <- matrix(predict(mod, newdata = dat), grid.lines, 
                  grid.lines)
                rgl.surface(vals, vals, yhat, color = surface.col[i], 
                  alpha = 0.5, lit = FALSE)
                if (grid) 
                  rgl.surface(vals, vals, yhat, color = grid.col, 
                    alpha = 0.5, lit = FALSE, front = "lines", 
                    back = "lines")
                if (residuals) {
                  n <- length(y)
                  if (coef.ratio != 1)
                    fitted <- predict(mod) ## new calculation
                  else
                    fitted <- fitted(mod) ## component of mod object
                  colors <- ifelse(residuals(mod) > 0, pos.res.col, 
                    neg.res.col)
                  if (squares)
                    rgl.quads(as.vector(rbind(x, x,
                                              x+(y-fitted), x+(y-fitted))),
                              as.vector(rbind(y, fitted, fitted, y)),
                              as.vector(rbind(z, z, z, z)),
                              color = square.color)
                  rgl.lines(as.vector(rbind(x, x)),
                            as.vector(rbind(y, fitted)),
                            as.vector(rbind(z, z)),
                            color = as.vector(rbind(colors, colors)))
                }
            }
            else {
                if (parallel) {
                  mod <- switch(f, linear = lm(y ~ x + z + groups), 
                    quadratic = lm(y ~ (x + z)^2 + I(x^2) + I(z^2) + 
                      groups), smooth = if (is.null(df.smooth)) 
                      gam(y ~ s(x, z) + groups)
                    else gam(y ~ s(x, z, fx = TRUE, k = df.smooth) + 
                      groups), additive = if (is.null(df.additive)) 
                      gam(y ~ s(x) + s(z) + groups)
                    else gam(y ~ s(x, fx = TRUE, k = df.additive[1] + 
                      1) + s(z, fx = TRUE, k = (rev(df.additive + 
                      1)[1] + 1)) + groups))
                  if (model.summary) 
                    summaries[[f]] <- summary(mod)
                  levs <- levels(groups)
                  for (j in 1:length(levs)) {
                    group <- levs[j]
                    select.obs <- groups == group
                    yhat <- matrix(predict(mod, newdata = cbind(dat, 
                      groups = group)), grid.lines, grid.lines)
                    rgl.surface(vals, vals, yhat, color = surface.col[j], 
                      alpha = 0.5, lit = FALSE)
                    if (grid) 
                      rgl.surface(vals, vals, yhat, color = grid.col, 
                        alpha = 0.5, lit = FALSE, front = "lines", 
                        back = "lines")
                    rgl.texts(0, predict(mod, newdata = data.frame(x = 0, 
                      z = 0, groups = group)), 0, paste(group, 
                      " "), adj = 1, color = surface.col[j])
                    if (residuals) {
                      yy <- y[select.obs]
                      xx <- x[select.obs]
                      zz <- z[select.obs]
                      fitted <- fitted(mod)[select.obs]
                      rgl.lines(as.vector(rbind(xx, xx)), as.vector(rbind(yy, 
                        fitted)), as.vector(rbind(zz, zz)), col = surface.col[j])
                    }
                  }
                }
                else {
                  levs <- levels(groups)
                  for (j in 1:length(levs)) {
                    group <- levs[j]
                    select.obs <- groups == group
                    mod <- switch(f, linear = lm(y ~ x + z, subset = select.obs), 
                      quadratic = lm(y ~ (x + z)^2 + I(x^2) + 
                        I(z^2), subset = select.obs), smooth = if (is.null(df.smooth)) 
                        gam(y ~ s(x, z), subset = select.obs)
                      else gam(y ~ s(x, z, fx = TRUE, k = df.smooth), 
                        subset = select.obs), additive = if (is.null(df.additive)) 
                        gam(y ~ s(x) + s(z), subset = select.obs)
                      else gam(y ~ s(x, fx = TRUE, k = df.additive[1] + 
                        1) + s(z, fx = TRUE, k = (rev(df.additive + 
                        1)[1] + 1)), subset = select.obs))
                    if (model.summary) 
                      summaries[[paste(f, ".", group, sep = "")]] <- summary(mod)
                    yhat <- matrix(predict(mod, newdata = dat), 
                      grid.lines, grid.lines)
                    rgl.surface(vals, vals, yhat, color = surface.col[j], 
                      alpha = 0.5, lit = FALSE)
                    rgl.surface(vals, vals, yhat, color = grid.col, 
                      alpha = 0.5, lit = FALSE, front = "lines", 
                      back = "lines")
                    rgl.texts(0, predict(mod, newdata = data.frame(x = 0, 
                      z = 0, groups = group)), 0, paste(group, 
                      " "), adj = 1, color = surface.col[j])
                    if (residuals) {
                      yy <- y[select.obs]
                      xx <- x[select.obs]
                      zz <- z[select.obs]
                      fitted <- fitted(mod)
                      rgl.lines(as.vector(rbind(xx, xx)), as.vector(rbind(yy, 
                        fitted)), as.vector(rbind(zz, zz)), col = surface.col[j])
                    }
                  }
                }
            }
        }
    }
    if (revolutions > 0) {
        for (i in 1:revolutions) {
            for (angle in seq(1, 360, length = 360/speed)) rgl.viewpoint(-angle, 
                fov = fov)
        }
    }
    if (model.summary) 
        return(summaries)
    else return(invisible(NULL))
}

"scatterPlotMatrix.HH" <-
function(){
    require("car")
    initializeDialog(title=gettextRcmdr("Scatterplot Matrix"))
    variablesBox <- variableListBox(top, Numeric(), title=gettextRcmdr("Select variables (three or more)"),
        selectmode="multiple", initialSelection=NULL)
    checkBoxes(frame="optionsFrame", boxes=c("lsLine", "smoothLine"), initialValues=rep(1,1),
        labels=gettextRcmdr(c("Least-squares lines", "Smooth lines")))
    sliderValue <- tclVar("50")
    slider <- tkscale(optionsFrame, from=0, to=100, showvalue=TRUE, variable=sliderValue,
        resolution=5, orient="horizontal")
    radioButtons(name="diagonal", buttons=c("density", "histogram", "boxplot", "oned", "qqplot", "none"),
        labels=gettextRcmdr(c("Density plots", "Histograms", "Boxplots", "One-dimensional scatterplots", "Normal QQ plots", "Nothing (empty)")),
        title=gettextRcmdr("On Diagonal"))
    radioButtons(name="diagonalDirection", buttons=c("SW.NE", "NW.SE"),
        labels=gettextRcmdr(c("SW-NE ' / '", "NW-SE ' \\ '")),
        title=gettextRcmdr("Diagonal Direction"))
    subsetBox()
    onOK <- function(){
        variables <- getSelection(variablesBox)
        closeDialog()
        if (length(variables) < 3) {
            errorCondition(recall=scatterPlotMatrix.HH, message=gettextRcmdr("Fewer than 3 variable selected."))
            return()
            }
        line <- if("1" == tclvalue(lsLineVariable)) "lm" else "FALSE"
        smooth <- as.character("1" == tclvalue(smoothLineVariable))
        span <- as.numeric(tclvalue(sliderValue))
        diag <- as.character(tclvalue(diagonalVariable))
        diagDir <- as.character(tclvalue(diagonalDirectionVariable))
        subset <- tclvalue(subsetVariable)
        subset <- if (trim.blanks(subset) == gettextRcmdr("<all valid cases>")) "" 
            else paste(", subset=", subset, sep="")
        .activeDataSet <- ActiveDataSet()
        if (.groups == FALSE) {
          command <- paste("scatterplot.matrix(~", paste(variables, collapse="+"),
                           ", reg.line=", line, ", smooth=", smooth,
                           ", span=", span/100, ", diagonal = '", diag,
                           "', data=", .activeDataSet, subset,
                           ", row1attop=", diagDir=="NW.SE",
                           ")", sep="")
          logger(command)
          justDoIt(command)
        }
        else {
          command <- paste("scatterplot.matrix(~", paste(variables, collapse="+")," | ", .groups,
                           ", reg.line=", line, ", smooth=", smooth,
                           ", span=", span/100, ", diagonal= '", diag,
                           "', by.groups=", .linesByGroup,
                           ", data=", .activeDataSet, subset,
                           ", row1attop=", diagDir=="NW.SE",
                           ")", sep="")
          logger(command)
          justDoIt(command)
        }
        activateMenus()
        tkfocus(CommanderWindow())
        }
    groupsBox(scatterPlot, plotLinesByGroup=TRUE)
    OKCancelHelp(helpSubject="scatterplot.matrix")
    tkgrid(getFrame(variablesBox), sticky="nw")    
    tkgrid(optionsFrame, sticky="w")
    tkgrid(diagonalFrame, sticky="w")
    tkgrid(diagonalDirectionFrame, sticky="w")
    tkgrid(subsetFrame, sticky="w")
    tkgrid(groupsFrame, sticky="w")
    tkgrid(buttonsFrame, columnspan=2, sticky="w")
    dialogSuffix(rows=6, columns=2)
    }

scatterPlot.HH <- function(){
    require("car")
    initializeDialog(title=gettextRcmdr("Scatterplot.HH"))
    .numeric <- Numeric()
    variablesFrame <- tkframe(top)
    xBox <- variableListBox(variablesFrame, .numeric, title=gettextRcmdr("x-variable (pick one)"))
    yBox <- variableListBox(variablesFrame, .numeric, title=gettextRcmdr("y-variable (pick one)"))
    checkBoxes(frame="optionsFrame", boxes=c("identify", "jitterX", "jitterY", "boxplots", "lsLine", "smoothLine"),
        initialValues=c(0, 0, 0, 0, 1, 0), labels=gettextRcmdr(c("Identify points", "Jitter x-variable", "Jitter y-variable",
        "Marginal boxplots", "Least-squares line", "Smooth Line")))
    sliderValue <- tclVar("50")
    slider <- tkscale(optionsFrame, from=0, to=100, showvalue=TRUE, variable=sliderValue,
        resolution=5, orient="horizontal")
    subsetBox()    
    labelsFrame <- tkframe(top)
    xlabVar <- tclVar(gettextRcmdr("<auto>"))
    ylabVar <- tclVar(gettextRcmdr("<auto>"))
    xlabFrame <- tkframe(labelsFrame)
    xlabEntry <- tkentry(xlabFrame, width="25", textvariable=xlabVar)
    xlabScroll <- tkscrollbar(xlabFrame, orient="horizontal",
        repeatinterval=5, command=function(...) tkxview(xlabEntry, ...))
    tkconfigure(xlabEntry, xscrollcommand=function(...) tkset(xlabScroll, ...))
    tkgrid(tklabel(xlabFrame, text=gettextRcmdr("x-axis label"), fg="blue"), sticky="w")
    tkgrid(xlabEntry, sticky="w")
    tkgrid(xlabScroll, sticky="ew")
    ylabFrame <- tkframe(labelsFrame)
    ylabEntry <- tkentry(ylabFrame, width="25", textvariable=ylabVar)
    ylabScroll <- tkscrollbar(ylabFrame, orient="horizontal",
        repeatinterval=5, command=function(...) tkxview(ylabEntry, ...))
    tkconfigure(ylabEntry, xscrollcommand=function(...) tkset(ylabScroll, ...))
    tkgrid(tklabel(ylabFrame, text=gettextRcmdr("y-axis label"), fg="blue"), sticky="w")
    tkgrid(ylabEntry, sticky="w")
    tkgrid(ylabScroll, sticky="ew")
    tkgrid(xlabFrame, tklabel(labelsFrame, text="     "), ylabFrame, sticky="w")    
    parFrame <- tkframe(top) 
    pchVar <- tclVar(gettextRcmdr("<auto>"))
    pchEntry <- tkentry(parFrame, width=25, textvariable=pchVar)      
    cexValue <- tclVar("1.3")
    cex.axisValue <- tclVar("1.3")
    cex.labValue <- tclVar("1.3")    
    cexSlider <- tkscale(parFrame, from=0.5, to=2.5, showvalue=TRUE, variable=cexValue,
        resolution=0.1, orient="horizontal")
    cex.axisSlider <- tkscale(parFrame, from=0.5, to=2.5, showvalue=TRUE, variable=cex.axisValue,
        resolution=0.1, orient="horizontal")
    cex.labSlider <- tkscale(parFrame, from=0.5, to=2.5, showvalue=TRUE, variable=cex.labValue,
        resolution=0.1, orient="horizontal")
    onOK <- function(){
        x <- getSelection(xBox)
        y <- getSelection(yBox)
        closeDialog()
        if (length(x) == 0 || length(y) == 0){
            errorCondition(recall=scatterPlot.HH, message=gettextRcmdr("You must select two variables"))
            return()
            }
        if (x == y) {
            errorCondition(recall=scatterPlot.HH, message=gettextRcmdr("x and y variables must be different"))
            return()
            }
        .activeDataSet <- ActiveDataSet()
        jitter <- if ("1" == tclvalue(jitterXVariable) && "1" == tclvalue(jitterYVariable)) ", jitter=list(x=1, y=1)"
            else if ("1" == tclvalue(jitterXVariable)) ", jitter=list(x=1)"
            else if ("1" == tclvalue(jitterYVariable)) ", jitter=list(y=1)"
            else ""
        if("1" == tclvalue(identifyVariable)){
            RcmdrTkmessageBox(title="Identify Points",
                message=gettextRcmdr("Use left mouse button to identify points,\nright button to exit."),
                icon="info", type="ok")
            labels <- paste("rownames(", .activeDataSet, ")", sep="")
            }
        else labels <- "FALSE"
        box <- if ("1" == tclvalue(boxplotsVariable)) "'xy'" else "FALSE"
        line <- if("1" == tclvalue(lsLineVariable)) "lm" else "FALSE"
        smooth <- as.character("1" == tclvalue(smoothLineVariable))
        span <- as.numeric(tclvalue(sliderValue))
        subset <- tclvalue(subsetVariable)
        subset <- if (trim.blanks(subset) == gettextRcmdr("<all valid cases>")) "" 
            else paste(", subset=", subset, sep="")
        xlab <- trim.blanks(tclvalue(xlabVar))
        xlab <- if(xlab == gettextRcmdr("<auto>")) "" else paste(', xlab="', xlab, '"', sep="")
        ylab <- trim.blanks(tclvalue(ylabVar))
        ylab <- if(ylab == gettextRcmdr("<auto>")) "" else paste(', ylab="', ylab, '"', sep="")
        cex <- as.numeric(tclvalue(cexValue))
        cex <- if(cex == 1) "" else paste(', cex=', cex, sep="")
        cex.axis <- as.numeric(tclvalue(cex.axisValue))
        cex.axis <- if(cex.axis == 1) "" else paste(', cex.axis=', cex.axis, sep="")
        cex.lab <- as.numeric(tclvalue(cex.labValue))
        cex.lab <- if(cex.lab == 1) "" else paste(', cex.lab=', cex.lab, sep="")        
        pch <- gsub(" ", ",", tclvalue(pchVar))
        if ("" == pch) {
            errorCondition(recall=scatterPlot.HH, message=gettextRcmdr("No plotting characters."))
            return()
            }
        pch <- if(trim.blanks(pch) == gettextRcmdr("<auto>")) {
          if (.groups == FALSE) ", pch=16" else ""
        }
        else paste(", pch=c(", pch, ")", sep="")
        if (.groups == FALSE) {
            doItAndPrint(paste("scatterplot(", y, "~", x,
                ", reg.line=", line, ", smooth=", smooth, ", labels=", labels,
                ", boxplots=", box, ", span=", span/100, jitter, xlab, ylab,
                cex, cex.axis, cex.lab, pch,
                ", data=", .activeDataSet, subset, ")", sep=""))
            }
        else {
            doItAndPrint(paste("scatterplot(", y, "~", x," | ", .groups,
                ", reg.line=", line, ", smooth=", smooth, ", labels=", labels,
                ", boxplots=", box, ", span=", span/100, jitter, xlab, ylab,
                cex, cex.axis, cex.lab, pch,
                ", by.groups=", .linesByGroup,
                ", data=", .activeDataSet, subset, ")", sep=""))
            }
        activateMenus()
        tkfocus(CommanderWindow())
        }
    groupsBox(scatterPlot, plotLinesByGroup=TRUE)
    OKCancelHelp(helpSubject="scatterplot")
    tkgrid(getFrame(xBox), getFrame(yBox), sticky="nw") 
    tkgrid(variablesFrame, sticky="w")   
    tkgrid(tklabel(optionsFrame, text=gettextRcmdr("Span for smooth")), slider, sticky="w")
    tkgrid(optionsFrame, sticky="w")
    tkgrid(subsetFrame, sticky="w")
    tkgrid(groupsFrame, sticky="w")    
    tkgrid(labelsFrame, sticky="w")
    tkgrid(tklabel(top, text=" "))    
    tkgrid(tklabel(parFrame, text=gettextRcmdr("Plotting Parameters"), fg="blue"), sticky="w")
    tkgrid(tklabel(parFrame, text=gettextRcmdr("Plotting characters")), pchEntry, stick="w")
    tkgrid(tklabel(parFrame, text=gettextRcmdr("Point size")), cexSlider, sticky="w")
    tkgrid(tklabel(parFrame, text=gettextRcmdr("Axis text size")), cex.axisSlider, sticky="w")
    tkgrid(tklabel(parFrame, text=gettextRcmdr("Axis-labels text size")), cex.labSlider, sticky="w")
    tkgrid(parFrame, sticky="w")
    tkgrid(buttonsFrame, columnspan=2, sticky="w")
    dialogSuffix(rows=8, columns=2)
    }

## source("~/HH-R.package/RcmdrPlugin.HH/R/scatterplot.HH.R")
"twoWayTable.HH" <-
function(){
    require("abind")
    initializeDialog(title=gettextRcmdr("Two-Way Table"))
    variablesFrame <- tkframe(top)
    .factors <- Factors()
    rowBox <- variableListBox(variablesFrame, .factors, title=gettextRcmdr("Row variable (pick one)"))
    columnBox <- variableListBox(variablesFrame, .factors, title=gettextRcmdr("Column variable (pick one)"))
    subsetBox()
    onOK <- function() {
        row <- getSelection(rowBox)
        column <- getSelection(columnBox)
        if (length(row) == 0 || length(column) == 0){
            errorCondition(recall=twoWayTable.HH, message=gettextRcmdr("You must select two variables."))
            return()
            }
        if (row == column) {
            errorCondition(recall=twoWayTable.HH, message=gettextRcmdr("Row and column variables are the same."))
            return()
            }        

        rowPct <- tclvalue(rowPercentsVariable)
        colPct <- tclvalue(colPercentsVariable)
        totPct <- tclvalue(totPercentsVariable)
##        percents <- as.character(tclvalue(percentsVariable))
        chisq <- tclvalue(chisqTestVariable)
        chisqComp <- tclvalue(chisqCompVariable)
        chiComp <- tclvalue(chiCompVariable)
        expected <- tclvalue(expFreqVariable)
        fisher <- tclvalue(fisherTestVariable)

        subset <- tclvalue(subsetVariable)
        subset <- if (trim.blanks(subset) == gettextRcmdr("<all valid cases>")) "" 
            else paste(", subset=", subset, sep="")
        closeDialog()

        command <- paste("xtabs(~", row, "+", column, ", data=", ActiveDataSet(),
            subset, ")", sep="")
        logger(paste(".Table <- ", command, sep=""))
        assign(".Table", justDoIt(command), envir=.GlobalEnv)
        doItAndPrint(".Table")

        if (rowPct == 1) doItAndPrint("rowPercents(.Table) # Row Percentages")
        if (colPct == 1) doItAndPrint("colPercents(.Table) # Column Percentages")        
        if (totPct == 1) doItAndPrint("totPercents2.HH(.Table) # Total Percentages")        

        if (chisq == 1) {
            command <- "chisq.test(.Table, correct=FALSE)"
            logger(paste(".Test <- ", command, sep=""))
            assign(".Test", justDoIt(command), envir=.GlobalEnv)
            doItAndPrint(".Test")
            if (chisqComp == 1) doItAndPrint("round(.Test$residuals^2, 2) # Chi-square Components")
            if (chiComp == 1) doItAndPrint("round(.Test$residuals, 2) # Chi Components (residuals)")
            if (expected == 1) doItAndPrint("round(.Test$expected, 2) # Expected Counts")
##            if (expected == 1) doItAndPrint(".Test$expected # Expected Counts")
            warnText <- NULL
            if (0 < (nlt1 <- sum(.Test$expected < 1))) warnText <- paste(nlt1,
                gettextRcmdr("expected frequencies are less than 1"))
            if (0 < (nlt5 <- sum(.Test$expected < 5))) warnText <- paste(warnText, "\n", nlt5,
                gettextRcmdr(" expected frequencies are less than 5"), sep="")
            if (!is.null(warnText)) Message(message=warnText,
                type="warning")
            logger("remove(.Test)") 
            remove(.Test, envir=.GlobalEnv) 
            }
        if (fisher == 1) doItAndPrint("fisher.test(.Table)")
        logger("remove(.Table)") 
        remove(.Table, envir=.GlobalEnv)                                                      
        tkfocus(CommanderWindow())
        }
    OKCancelHelp(helpSubject="xtabs")

    checkBoxes(frame="percentsFrame",
               boxes=c("rowPercents",
                 "colPercents",
                 "totPercents"), 
               initialValues=c("0","0","0"), 
               labels=gettextRcmdr(c(
                 "Row percentages",
                 "Column percentages",
                 "Total percentages")))
    
##   radioButtons(name="percents",
##                buttons=c("rowPercents",
##                  "columnPercents",
##                  "nonePercents"), 
##                values=c("row", "column", "none"),
##                initialValue="none", 
##                labels=gettextRcmdr(c
##                  ("Row percentages",
##                   "Column percentages",
##                   "No percentages")),
##                title=gettextRcmdr("Compute Percentages"))
    
    checkBoxes(frame="testsFrame",
               boxes=c(
                 "chisqTest",
                 "chisqComp",
                 "chiComp",
                 "expFreq",
                 "fisherTest"),
               initialValues=c("1", "1", "0", "0", "0"),
               labels=gettextRcmdr(c(
                 "Chi-square test of independence",
                 "Print chi-square components",
                 "Print chi components (residuals)",
                 "Print expected frequencies",
                 "Fisher's exact test")))

##     checkBoxes(frame="testsFrame",
##                boxes=c("chisqTest", "expFreq", "fisherTest"), initialValues=c("1", "0", "0"),
##                labels=gettextRcmdr(
##                  c("Chi-square test of independence",
##                    "Print expected frequencies",
##                    "Fisher's exact test")))

    tkgrid(getFrame(rowBox),
           tklabel(variablesFrame, text="    "),
           getFrame(columnBox), sticky="nw")
    tkgrid(variablesFrame, sticky="w")
    tkgrid(tklabel(top, text=gettextRcmdr("Compute Percentages"), fg="blue"), sticky="w")
    tkgrid(percentsFrame, sticky="w")
    tkgrid(tklabel(top,
                   text=gettextRcmdr("Hypothesis Tests"),
                   fg="blue"),
           sticky="w")
    tkgrid(testsFrame, sticky="w")
    tkgrid(subsetFrame, sticky="w")
    tkgrid(buttonsFrame, sticky="w")
    dialogSuffix(rows=6, columns=1)
  }

.First.lib <- function(libname, pkgname){
  if (!interactive()) return()
  Rcmdr <- options()$Rcmdr
  plugins <- Rcmdr$plugins
  if ((!pkgname %in% plugins) && !getRcmdr("autoRestart")) {
    Rcmdr$plugins <- c(plugins, pkgname)
    options(Rcmdr=Rcmdr)
    closeCommander(ask=FALSE, ask.save=TRUE)
    Commander()
  }
}

